module ccrosskolmog

	use chelix
	implicit none
	private

	integer                             :: nxpad
	complex, allocatable, dimension (:) :: tmp1, tmp2

	public kolfactorchelix_alloc, kolfactorchelix_dealloc, kolfactorchelix

contains
	subroutine kolfactorchelix_alloc(nxpad_in)
		integer :: nxpad_in,pad2

		nxpad=pad2(nxpad_in)

		write(0,*) 'nxpad=',nxpad
		allocate (tmp1(nxpad),tmp2(nxpad))

	end subroutine kolfactorchelix_alloc

	subroutine kolfactorchelix_dealloc()
		deallocate(tmp1,tmp2)
	end subroutine kolfactorchelix_dealloc

	subroutine kolfactorchelix(cross,fac1,fac2) ! interface for xkolmog
		type (cfilter)		:: cross, fac1, fac2
		integer				:: iw,ifil,i,ith
		real*8				:: dw,w1

		dw=(8*atan(1.))/nxpad 
		tmp1(1:nxpad)=0.

		do iw=1, nxpad 
			 w1 = dw*(iw-1)
			 do ifil=1,size(cross%lag)
			 	tmp1(iw)=tmp1(iw)+cross%flt(ifil)*cexp(cmplx(0,cross%lag(ifil)*w1))
			 end do
			 tmp1(iw)=tmp1(iw)/(real(nxpad))
		end do

		call xkolmog(nxpad, tmp1(1:nxpad),tmp2(1:nxpad))  ! Kolmogoroff x-spectral factorization

		do i=1,size(fac1%flt)
			fac1%flt(i)=tmp1(fac1%lag(i)+1)
			fac2%flt(i)=tmp2(fac2%lag(i)+1)
		end do 
	end subroutine kolfactorchelix

	subroutine xkolmog( n, cx1, cx2)  ! Spectral factorization.
		integer i, n				! input:  cx1 = spectrum
		complex cx1(n),cx2(n)		! output: cx1, cx2 = 2 min phase causal wavelet
									! such that spectrum = cx2' * cx1

		cx1(1:n) = clog( cx1(1:n) )

		! go to time domain
		call jftu( -1., n, cx1)

		cx1(1    )=cx1(1    )/2.; cx2(1    )=cx1(1    )
		cx1(1+n/2)=cx1(1+n/2)/2.; cx2(1+n/2)=cx1(1+n/2)

		do i= n/2+2, n
			cx2(n-i+2) = cx1(i)
			cx1(i)=0.
			cx2(i)=0.
		end do
 
		! go to frequency domain
		call jftu( +1., n, cx1)
		call jftu( +1., n, cx2)

		do i= 1, n
			cx1(i) = cexp( cx1(i))
			cx2(i) = cexp( cx2(i))
		end do

		! back to time to finish
		call jftu( -1., n, cx1)
		call jftu( -1., n, cx2)

		cx2(1:n) = conjg(cx2(1:n))
	end subroutine xkolmog

	subroutine jftu( signi, nx, cx )
!   complex fast fourier transform with unitary scaling
!
!               1         nx          signi*2*pi*i*(j-1)*(k-1)/nx
!   cx(k)  =  -------- * sum cx(j) * e
!             sqrt(nx)   j=1             for k=1,2,...,nx=2**integer
!
		integer nx, i, j, k, m, istep, pad2
		real    signi, arg
		complex cx(nx), cmplx, cw, cdel, ct

		if( nx /= pad2(nx) )  call erexit('ftu: nx not a power of 2')
		cx = cx/sqrt( 1.*nx)
		j = 1
		k = 1

		do i= 1, nx
			if (i<=j) then
				  ct = cx(j); cx(j) = cx(i); cx(i) = ct
			end if
			m = nx/2
			do while (j>m .and. m>1) 
				j = j-m; m = m/2 
			end do
			j = j+m
		end do

		do 
			istep = 2*k
			cw = 1.
			arg = signi*3.14159265/k

			cdel = cmplx( cos(arg), sin(arg))
			do m= 1, k
				do i= m, nx, istep
					ct=cw*cx(i+k);  cx(i+k)=cx(i)-ct;  cx(i)=cx(i)+ct
				end do
				cw = cw * cdel
			end do
			k = istep
			if (k>=nx) exit
		end do
	end subroutine jftu

end module ccrosskolmog



