module chelix  
  ! DEFINE complex helix filter type
  implicit none
  type cfilter  
     complex, dimension(:), pointer :: flt      ! (nh) filter coefficients
     integer, dimension(:), pointer :: lag      ! (nh) filter lags
     logical, dimension(:), pointer :: mis      ! (nd) boundary conditions
  end type cfilter
contains

  ! allocate a filter
  subroutine allocatechelix( aa, nh )
    type( cfilter) :: aa
    integer       :: nh                         ! no. of filter coefs (excl 1)
    allocate( aa%flt( nh), aa%lag( nh))         ! allocate filter and lags.
    nullify( aa%mis)                            ! set null pointer for "mis".
    aa%flt = 0.                                 ! zero filter coef values
  end subroutine allocatechelix

  ! destroy a filter
  subroutine deallocatechelix( aa)
    type( cfilter) :: aa
    deallocate( aa%flt, aa%lag)                 ! free memory
    ! if logicals were allocated
    if ( associated( aa%mis)) then
      deallocate( aa%mis)                       ! free them
    end if
  end subroutine deallocatechelix
end module 
