% Von-Neuman stability of 1D implicit FD vs. explicit as function of dt

% Yaxuns FD weights
%subroutine rtm_coeffs_init(cc, nop)
%    integer              :: nop
%    real, dimension(nop) :: cc
%    select case(nop)
%       case(1)
%         cc(1) = 1.
%       case(2)
%         cc(1) = 16./12.; cc(2) = -1./12.
%       case(3)
%         cc(1) =  3./ 2.; cc(2) = -3./20.; cc(3) = 1./90.
%       case(4)
%         cc(1) =  1.6   ; cc(2) = -0.2   ; cc(3) = 8./315.; cc(4) = -1./560.
%       case(5)
%         cc(1) =  5./ 3.; cc(2) = -5./21.; cc(3) = 5./126.; cc(4) = -5./1008.; cc(5) = 1./3150.
%    end select
%    return
%  end subroutine


echo off;
clear;
clc;

%load('blue_red_stab', 'blue_red_stab_map')
set(0,'DefaultAxesFontSize',18);
set(0,'DefaultFigureColorMap',jet);
set(0, 'DefaultFigurePosition', [1 1 1600 950]);

nx	= 200;
dx  = 10;
v   = 1000;
dt  = 0.001:0.001:0.1;
%dt  = 0.01:0.01:0.2;
mdt = dt .* 1e+3;

dt_first = mdt(1);
mdt_last = mdt(size(mdt(:)));
dt_last = mdt_last(1);

disp_max = 1.0;
disp_min = 0.;

v2  = v^2;
dx2 = dx^2;

w2_0	= -2.;
w2_1	=  1.;

w4_0	= -30. / 12.;
w4_1	=  16. / 12.;
w4_2	= -1.  / 12.;

w6_0	= -2.72222222222;
w6_1	=  3. / 2.;
w6_2	= -3. / 20.;
w6_3	=  1. / 90.;

w8_0	= -2.84722222222;
w8_1	=  8. / 5.;
w8_2	= -1. / 5.;
w8_3	=  8. / 315.;
w8_4	= -1. / 560.;

w10_0	= -7383. / 2520.;
w10_1	=  5. / 3.;
w10_2	= -5. / 21.;
w10_3	=  5. / 126.;
w10_4	= -5. / 1008.;
w10_5	=  5. / 3150.;

k		= 0 : pi/(nx*dx) : pi;
kdx		= k * dx;
kabs	= abs(k);
k_first = k(1);
k_last = k(size(k(:)));
k_last = k_last(1);

alpha2	 = v2 .* dt.^2 / (4*dx2);
alphaexp = v2 .* dt.^2 / dx2;

cos_k1	= cos(1. * kdx);
cos_k2	= cos(2. * kdx);
cos_k3	= cos(3. * kdx);
cos_k4	= cos(4. * kdx);
cos_k5	= cos(5. * kdx);

eps		= 0.1;

R2  = w2_1 .* cos_k1 + w2_0 / 2.;
R4  = w4_2 .* cos_k2 + w4_1 .* cos_k1 + w4_0 / 2.;
R6  = w6_3 .* cos_k3 + w6_2 .* cos_k2 + w6_1 .* cos_k1 + w6_0 / 2.;
R8  = w8_4 .* cos_k4 + w8_3 .* cos_k3 + w8_2 .* cos_k2 + w8_1 .* cos_k1 + w8_0 / 2.;
R10 = w10_5 .* cos_k5 + w10_4 .* cos_k4 + w10_3 .* cos_k3 + w10_2 .* cos_k2 + w10_1 .* cos_k1 + w10_0 / 2.;

for i = 1:size(dt(:))

	% Implicit schemes
	stab_imp2(i,:)  = abs ((1. + 2 * alpha2(i) .* R2)  ./ (1. - 2 * alpha2(i) .* R2));
	stab_imp4(i,:)  = abs ((1. + 2 * alpha2(i) .* R4)  ./ (1. - 2 * alpha2(i) .* R4));
	stab_imp6(i,:)  = abs ((1. + 2 * alpha2(i) .* R6)  ./ (1. - 2 * alpha2(i) .* R6));
	stab_imp8(i,:)  = abs ((1. + 2 * alpha2(i) .* R8)  ./ (1. - 2 * alpha2(i) .* R8));
	stab_imp10(i,:) = abs ((1. + 2 * alpha2(i) .* R10) ./ (1. - 2 * alpha2(i) .* R10));

	stab_exp2(i,:)  = abs ((1. + alphaexp(i) .* R2));
	stab_exp4(i,:)  = abs ((1. + alphaexp(i) .* R4));
	stab_exp6(i,:)  = abs ((1. + alphaexp(i) .* R6));
	stab_exp8(i,:)  = abs ((1. + alphaexp(i) .* R8));
	stab_exp10(i,:) = abs ((1. + alphaexp(i) .* R10));

	% Explicit schemes
end

stab_imp2 = (stab_imp2 > 1.0001) * 2 + (stab_imp2 < 1.0001) * 0;
stab_imp4 = (stab_imp2 > 1.0001) * 2 + (stab_imp4 < 1.0001) * 0;
stab_imp10 = (stab_imp2 > 1.0001) * 2 + (stab_imp10 < 1.0001) * 0;
stab_exp2 = (stab_exp2 > 1.0001) * 2 + (stab_exp2 < 1.0001) * 0;
stab_exp4 = (stab_exp2 > 1.0001) * 2 + (stab_exp4 < 1.0001) * 0;
stab_exp10 = (stab_exp2 > 1.0001) * 2 + (stab_exp10 < 1.0001) * 0;


cax_min = disp_min;
%cax_max = disp_max;
%cax_min = min(min(Vp2));
cax_max = max(max(stab_exp2));


tit_imp2 = sprintf ('2nd order');
tit_imp4 = sprintf ('4th order');
tit_imp10 = sprintf ('10th order');

tit_exp2 = sprintf ('2nd order');
tit_exp4 = sprintf ('4th order');
tit_exp10 = sprintf ('10th order');

%display explicit dispersion relations


subplot (3,2,1); surf (kdx, mdt, stab_exp2); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_exp2);
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt (ms)');

subplot (3,2,2); surf (kdx, mdt, stab_imp2); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_imp2);
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt (ms)');

subplot (3,2,3); surf (kdx, mdt, stab_exp4); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_exp4);
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt (ms)');

subplot (3,2,4); surf (kdx, mdt, stab_imp4); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_imp4);
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt (ms)');

subplot (3,2,5); surf (kdx, mdt, stab_exp10); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_exp10);
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt (ms)');

subplot (3,2,6); surf (kdx, mdt, stab_imp10); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_imp10);
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt (ms)');
