% dispersion relation for 1D implicit FD vs. explicit as function of dt
echo off;
clear;
clc;

nx	= 100;
dx  = 10;
v   = 1000;
dt  = 0.001:0.001:0.02;

v2  = v^2;
dx2 = dx^2;

k		= pi/(nx*dx) : pi/(nx*dx) : pi/(2*dx);
kdx		= k * dx;
kabs	= abs(k);
alpha	= v2 .* dt.^2 / (3*dx2);
alpha2	= v2 .* dt.^2 / (4*dx2);
alphaexp = v * dt / dx;
sin_k	= sin(kdx ./ 2);
eps		= 0.1;

for i = 1:size(dt(:))

	imp_asin	= asin ( sqrt(3*alpha(i)) .* sin_k ./ sqrt (1 + 4 .*alpha(i) .* (sin_k).^2 ));
	Vp(i,:)		= 2 .* imp_asin ./ (dt(i) .* kabs .* v);

	imp_asin2	= asin ( sqrt((4+eps)*alpha2(i)) .* sin_k ./ sqrt (1 + 4 .*alpha2(i) .* (sin_k).^2 ));
	Vp2(i,:)	= 2 .* imp_asin2 ./ (dt(i) .* kabs .* v);

	exp_asin	= alphaexp(i) .* sin_k;

	for j = 1 : size(sin_k(:))
		exp_asin2 = asin (exp_asin(j));
		if (abs(exp_asin2) > 1.0)
			%Vpexp(i,j) = 1.0;
			Vpexp(i,j) = max(max(Vpexp));
		else
			Vpexp(i,j) = 2 * exp_asin2 / (dt(i) * kabs(j) * v);
		end
	end

end

figure
title ('implicit dispersion relation normalized by medium velocity');
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');
surf (kdx, dt, Vp);
view (0, 90);
%legend ('Vphase / Vmedium');
colorbar;

%figure
%title ('implicit2 dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');
%surf (kdx, dt, Vp2);
%view (0, 90);
%legend ('Vphase / Vmedium');
%colorbar;

%figure
%title ('implicit dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');
%surf (kdx, dt, Vp2./Vp);
%view (0, 90);
%legend ('Vphase / Vmedium');
%colorbar;

%figure
%title ('explicit dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');
%surf (kdx, dt, Vpexp);
%view (0, 90);
%legend ('Vphase / Vmedium');
%colorbar;
