% dispersion relation for 1D implicit FD vs. explicit as function of dt

% Yaxuns FD weights
%subroutine rtm_coeffs_init(cc, nop)
%    integer              :: nop
%    real, dimension(nop) :: cc
%    select case(nop)
%       case(1)
%         cc(1) = 1.
%       case(2)
%         cc(1) = 16./12.; cc(2) = -1./12.
%       case(3)
%         cc(1) =  3./ 2.; cc(2) = -3./20.; cc(3) = 1./90.
%       case(4)
%         cc(1) =  1.6   ; cc(2) = -0.2   ; cc(3) = 8./315.; cc(4) = -1./560.
%       case(5)
%         cc(1) =  5./ 3.; cc(2) = -5./21.; cc(3) = 5./126.; cc(4) = -5./1008.; cc(5) = 1./3150.
%    end select
%    return
%  end subroutine


echo off;
clear;
clc;

nx	= 500;
dx  = 10;
v   = 1000;
dt  = 0.001:0.0005:0.05;
dt_first = dt(1);
dt_last = dt(size(dt(:)));
dt_last = dt_last(1);

%dt  = 0.01:0.01:0.2;

v2  = v^2;
dx2 = dx^2;

w4_0	= -30. / 12.;
w4_1	=  16. / 12.;
w4_2	= -1.  / 12.;

w6_0	= -2.72222222222;
w6_1	=  3. / 2.;
w6_2	= -3. / 20.;
w6_3	=  1. / 90.;

w8_0	= -2.84722222222;
w8_1	=  8. / 5.;
w8_2	= -1. / 5.;
w8_3	=  8. / 315.;
w8_4	= -1. / 560.;

w10_0	= -7383. / 2520.;
w10_1	=  5. / 3.;
w10_2	= -5. / 21.;
w10_3	=  5. / 126.;
w10_4	= -5. / 1008.;
w10_5	=  5. / 3150.;
%w10_0	= -5.8544444444;
%w10_1	=  3.3333333333;
%w10_2	= -0.4761904762;
%w10_3	=  0.0793650794;
%w10_4	= -0.0099206349;
%w10_5	=  0.0006349206;

k		= pi/(nx*dx) : pi/(nx*dx) : pi/(2*dx);
kdx		= k * dx;
kabs	= abs(k);
alpha	= v2 .* dt.^2 / (3*dx2);
alpha2	= v2 .* dt.^2 / (4*dx2);
alphaexp = v2 * dt.^2 / dx2;
sin_k1	= sin(0.5 * kdx) .^2;
sin_k2	= sin(1.0 * kdx) .^2;
sin_k3	= sin(1.5 * kdx) .^2;
sin_k4	= sin(2.0 * kdx) .^2;
sin_k5	= sin(2.5 * kdx) .^2;
eps		= 0.1;

for i = 1:size(dt(:))

	% Implicit schemes

	imp_asin	= asin ( sqrt(3*alpha(i) .* sin_k1 ./ (1 + 4 .*alpha(i) .* sin_k1 )) );
	Vp(i,:)		= 2 .* imp_asin ./ (dt(i) .* kabs .* v);

	%imp_asin2	= asin ( sqrt((4+eps)*alpha2(i) .* sin_k1 ./ (1 + 4 .*alpha2(i) .* sin_k1 )) );
	imp_asin2	= asin ( sqrt(4*alpha2(i) .* sin_k1 ./ (1 + 4 .*alpha2(i) .* sin_k1 )) );
	Vp2(i,:)	= 2 .* imp_asin2 ./ (dt(i) .* kabs .* v);


	imp_asin4	= asin (sqrt (3*alpha(i) .* (w4_2 .* sin_k2 + w4_1 .* sin_k1) ./ ...
 					   (1 + 4 .* alpha(i) .* (w4_2 .* sin_k2 + w4_1 .* sin_k1) )) );
	Vp4(i,:)	= 2 .* imp_asin4 ./ (dt(i) .* kabs .* v);

	imp_asin42	= asin (sqrt (4*alpha2(i) .* (w4_2 .* sin_k2 + w4_1 .* sin_k1) ./ ...
					   (1 + 4 .* alpha2(i) .* (w4_2 .* sin_k2 + w4_1 .* sin_k1) )) );
	Vp42(i,:)	= 2 .* imp_asin42 ./ (dt(i) .* kabs .* v);

	imp_asin10	= asin (sqrt (3*alpha(i) .* (w10_5 .* sin_k5 + w10_4 .* sin_k4 + w10_3 .* sin_k3 + w10_2 .* sin_k2 + w10_1 .* sin_k1) ./ ...
 					   (1 + 4 .* alpha(i) .* (w10_5 .* sin_k5 + w10_4 .* sin_k4 + w10_3 .* sin_k3 + w10_2 .* sin_k2 + w10_1 .* sin_k1) )) );
	Vp10(i,:)	= 2 .* imp_asin10 ./ (dt(i) .* kabs .* v);

	imp_asin102	= asin (sqrt (4*alpha2(i) .* (w10_5 .* sin_k5 + w10_4 .* sin_k4 + w10_3 .* sin_k3 + w10_2 .* sin_k2 + w10_1 .* sin_k1) ./ ...
 					   (1 + 4 .* alpha2(i) .* (w10_5 .* sin_k5 + w10_4 .* sin_k4 + w10_3 .* sin_k3 + w10_2 .* sin_k2 + w10_1 .* sin_k1) )) );
	Vp102(i,:)	= 2 .* imp_asin102 ./ (dt(i) .* kabs .* v);

	% Explicit schemes

	exp_sqrt	= sqrt (alphaexp(i) .* sin_k1);
	exp_sqrt4	= sqrt (alphaexp(i) .* (w4_2 .* sin_k2 + w4_1 .* sin_k1) );
	exp_sqrt10	= sqrt (alphaexp(i) .* (w10_5 .* sin_k5 + w10_4 .* sin_k4 ...
									  + w10_3 .* sin_k3 + w10_2 .* sin_k2 + w10_1 .* sin_k1) );
	for j = 1 : size(k(:))
		if (exp_sqrt(j) > 1.0)
			exp_sqrt(j) = 1.0;
		end
		exp_asin   = asin (exp_sqrt(j));
		Vpexp(i,j) = 2 * exp_asin ./ (dt(i) * kabs(j) * v);
		if (Vpexp(i,j) > 1.0)
			Vpexp(i,j) = 1.2;
		end

		if (exp_sqrt4(j) > 1.0)
			exp_sqrt4(j) = 1.0;
		end
		exp_asin4   = asin (exp_sqrt4(j));
		Vpexp4(i,j) = 2 * exp_asin4 ./ (dt(i) * kabs(j) * v);
		if (Vpexp4(i,j) > 1.0)
			Vpexp4(i,j) = 1.2;
		end

		if (exp_sqrt10(j) > 1.0)
			exp_sqrt10(j) = 1.0;
		end
		exp_asin10 = asin (exp_sqrt10(j));
		Vpexp10(i,j) = 2 * exp_asin10 ./ (dt(i) * kabs(j) * v);
		if (Vpexp10(i,j) > 1.0)
			Vpexp10(i,j) = 1.2;
		end
	end


end


%cax_min = min(min(Vp2));
%cax_max = max(max(Vpexp10));

cax_min = 0.4;
cax_max = 1.2;


figure
subplot (3,2,1); surf (kdx, dt, Vpexp); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([0 1.57 dt_first dt_last]);  shading interp;
title ('explicit 2nd order dispersion relation normalized by medium velocity');
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

subplot (3,2,2); surf (kdx, dt, Vp2); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([0 1.57 dt_first dt_last]);  shading interp;
title ('implicit 2nd order dispersion relation normalized by medium velocity');
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

subplot (3,2,3); surf (kdx, dt, Vpexp4); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([0 1.57 dt_first dt_last]);  shading interp;
title ('explicit 4th order dispersion relation normalized by medium velocity');
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

subplot (3,2,4); surf (kdx, dt, Vp42); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([0 1.57 dt_first dt_last]);  shading interp;
title ('implicit 4th order dispersion relation normalized by medium velocity');
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

subplot (3,2,5); surf (kdx, dt, Vpexp10); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([0 1.57 dt_first dt_last]);  shading interp;
title ('explicit 10th order dispersion relation normalized by medium velocity');
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

subplot (3,2,6); surf (kdx, dt, Vp102); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([0 1.57 dt_first dt_last]);  shading interp;
title ('implicit 10th order dispersion relation normalized by medium velocity');
xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

% these are comparisons between the double central weight schemes and the single central weight schemes

%figure
%subplot (3,2,1); surf (kdx, dt, Vp); view (0, 90); colorbar; caxis([cax_min cax_max]);
%title ('implicit dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

%subplot (3,2,2); surf (kdx, dt, Vp2); view (0, 90); colorbar; caxis([cax_min cax_max]);
%title ('implicit2 dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

%subplot (3,2,3); surf (kdx, dt, Vp4); view (0, 90); colorbar; caxis([cax_min cax_max]);
%title ('implicit4 dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

%subplot (3,2,4); surf (kdx, dt, Vp42); view (0, 90); colorbar; caxis([cax_min cax_max]);
%title ('implicit42 dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

%subplot (3,2,5); surf (kdx, dt, Vp10); view (0, 90); colorbar; caxis([cax_min cax_max]);
%title ('implicit10 dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

%subplot (3,2,6); surf (kdx, dt, Vp102); view (0, 90); colorbar; caxis([cax_min cax_max]);
%title ('implicit102 dispersion relation normalized by medium velocity');
%xlabel ('k*dx (rad)'), zlabel ('Vphase / Vmedium'), ylabel('dt');

%cax_min = min(min(Vpexp));
%cax_max = max(max(Vpexp10));
