% dispersion relation for 1D implicit FD vs. explicit as function of dt

% Yaxuns FD weights
%subroutine rtm_coeffs_init(cc, nop)
%    integer              :: nop
%    real, dimension(nop) :: cc
%    select case(nop)
%       case(1)
%         cc(1) = 1.
%       case(2)
%         cc(1) = 16./12.; cc(2) = -1./12.
%       case(3)
%         cc(1) =  3./ 2.; cc(2) = -3./20.; cc(3) = 1./90.
%       case(4)
%         cc(1) =  1.6   ; cc(2) = -0.2   ; cc(3) = 8./315.; cc(4) = -1./560.
%       case(5)
%         cc(1) =  5./ 3.; cc(2) = -5./21.; cc(3) = 5./126.; cc(4) = -5./1008.; cc(5) = 1./3150.
%    end select
%    return
%  end subroutine


echo off;
clear;
clc;

load('blue_red_disp', 'blue_red_disp_map')
set(0,'DefaultAxesFontSize',16);
set(0,'DefaultFigureColorMap',blue_red_disp_map);
set(0, 'DefaultFigurePosition', [1 1 1000 950]);
%set(0, 'DefaultAxesYTick', [1 5 10 15 20]);
set(0, 'DefaultAxesYTickLabelMode', 'auto');
set(0, 'DefaultAxesXTick', [0.785 1.57 2.355 3.14]);
set(0, 'DefaultAxesXTickLabel', {'pi/4' ; 'pi/2' ; '3*pi/4' ; 'pi'});

figure;

nx	= 200;
dx  = 10;
v   = 1000;
dt  = 0.0001:0.0001:0.02;
%dt  = 0.01:0.01:0.2;
mdt = dt .* 1e+3;

dt_first = mdt(1);
mdt_last = mdt(size(mdt(:)));
dt_last = mdt_last(1);

disp_max = 1.2;
disp_min = 0.5;

v2  = v^2;
dx2 = dx^2;

w4_0	= -30. / 12.;
w4_1	=  16. / 12.;
w4_2	= -1.  / 12.;

w6_0	= -2.72222222222;
w6_1	=  3. / 2.;
w6_2	= -3. / 20.;
w6_3	=  1. / 90.;

w8_0	= -2.84722222222;
w8_1	=  8. / 5.;
w8_2	= -1. / 5.;
w8_3	=  8. / 315.;
w8_4	= -1. / 560.;

w10_0	= -7383. / 2520.;
w10_1	=  5. / 3.;
w10_2	= -5. / 21.;
w10_3	=  5. / 126.;
w10_4	= -5. / 1008.;
w10_5	=  5. / 3150.;

%k		= pi/(nx*dx) : pi/(nx*dx) : pi/(2*dx);
k		= pi/(nx*dx) : pi/(nx*dx) : pi;
kdx		= k * dx;
kabs	= abs(k);
k_first = k(1);
k_last  = k(size(k(:)));
k_last  = k_last(1);

alpha	 = v2 .* dt.^2 / (3*dx2);
alpha2	 = v2 .* dt.^2 / (4*dx2);
alphaexp = v2 .* dt.^2 / dx2;

sin_k1	= sin(0.5 * kdx) .^2;
sin_k2	= sin(1.0 * kdx) .^2;
sin_k3	= sin(1.5 * kdx) .^2;
sin_k4	= sin(2.0 * kdx) .^2;
sin_k5	= sin(2.5 * kdx) .^2;

eps		= 0.1;

for i = 1:size(dt(:))

	% Implicit schemes

	%imp_asin	= asin ( sqrt((4+eps)*alpha2(i) .* sin_k1 ./ (1 + 4 .*alpha2(i) .* sin_k1 )) );
	imp_asin2	= asin ( sqrt(4*alpha2(i) .* sin_k1 ./ (1 + 4 .*alpha2(i) .* sin_k1 )) );
	Vp2(i,:)	= 2 .* imp_asin2 ./ (dt(i) .* kabs .* v);

	imp_asin4	= asin (sqrt (4*alpha2(i) .* (w4_2 .* sin_k2 + w4_1 .* sin_k1) ./ ...
					   (1 + 4 .* alpha2(i) .* (w4_2 .* sin_k2 + w4_1 .* sin_k1) )) );
	Vp4(i,:)	= 2 .* imp_asin4 ./ (dt(i) .* kabs .* v);

	imp_asin6	= asin (sqrt (4*alpha2(i) .* (w6_3 .* sin_k3 + w6_2 .* sin_k2 + w6_1 .* sin_k1) ./ ...
					   (1 + 4 .* alpha2(i) .* (w6_3 .* sin_k3 + w6_2 .* sin_k2 + w6_1 .* sin_k1) )) );
	Vp6(i,:)	= 2 .* imp_asin6 ./ (dt(i) .* kabs .* v);

	imp_asin8	= asin (sqrt (4*alpha2(i) .* (w8_4 .* sin_k4 + w8_3 .* sin_k3 + w8_2 .* sin_k2 + w8_1 .* sin_k1) ./ ...
					   (1 + 4 .* alpha2(i) .* (w8_4 .* sin_k4 + w8_3 .* sin_k3 + w8_2 .* sin_k2 + w8_1 .* sin_k1) )) );
	Vp8(i,:)	= 2 .* imp_asin8 ./ (dt(i) .* kabs .* v);

	imp_asin10	= asin (sqrt (4*alpha2(i) .* (w10_5 .* sin_k5 + w10_4 .* sin_k4 + w10_3 .* sin_k3 + w10_2 .* sin_k2 + w10_1 .* sin_k1) ./ ...
 					   (1 + 4 .* alpha2(i) .* (w10_5 .* sin_k5 + w10_4 .* sin_k4 + w10_3 .* sin_k3 + w10_2 .* sin_k2 + w10_1 .* sin_k1) )) );
	Vp10(i,:)	= 2 .* imp_asin10 ./ (dt(i) .* kabs .* v);

	% Explicit schemes

	exp_sqrt	= sqrt (alphaexp(i) .* sin_k1);
	exp_sqrt4	= sqrt (alphaexp(i) .* (w4_2  .* sin_k2 + w4_1  .* sin_k1) );
	exp_sqrt6	= sqrt (alphaexp(i) .* (w6_3  .* sin_k3 + w6_2  .* sin_k2 + w6_1  .* sin_k1) );
	exp_sqrt8	= sqrt (alphaexp(i) .* (w8_4  .* sin_k4 + w8_3  .* sin_k3 + w8_2  .* sin_k2 + w8_1  .* sin_k1) );
	exp_sqrt10	= sqrt (alphaexp(i) .* (w10_5 .* sin_k5 + w10_4 .* sin_k4 + w10_3 .* sin_k3 + w10_2 .* sin_k2 + w10_1 .* sin_k1) );


	for j = 1 : size(k(:))
		if (exp_sqrt(j) > 1.0) % argument of arcsine cannot be greater than 1
			exp_sqrt(j) = 1.0;
		end
		exp_asin   = asin (exp_sqrt(j));
		Vpexp(i,j) = 2 * exp_asin ./ (dt(i) * kabs(j) * v);

		if (exp_sqrt4(j) > 1.0)
			exp_sqrt4(j) = 1.0;
		end
		exp_asin4   = asin (exp_sqrt4(j));
		Vpexp4(i,j) = 2 * exp_asin4 ./ (dt(i) * kabs(j) * v);

		if (exp_sqrt6(j) > 1.0)
			exp_sqrt6(j) = 1.0;
		end
		exp_asin6   = asin (exp_sqrt6(j));
		Vpexp6(i,j) = 2 * exp_asin6 ./ (dt(i) * kabs(j) * v);

		if (exp_sqrt8(j) > 1.0)
			exp_sqrt8(j) = 1.0;
		end
		exp_asin8   = asin (exp_sqrt8(j));
		Vpexp8(i,j) = 2 * exp_asin8 ./ (dt(i) * kabs(j) * v);

		if (exp_sqrt10(j) > 1.0)
			exp_sqrt10(j) = 1.0;
		end
		exp_asin10 = asin (exp_sqrt10(j));
		Vpexp10(i,j) = 2 * exp_asin10 ./ (dt(i) * kabs(j) * v);
	end

end


%cax_min = disp_min;
cax_max = disp_max;
cax_min = min(min(Vp2));
%cax_max = max(max(Vpexp10));


tit_exp2 = sprintf ('');
tit_exp4 = sprintf ('');
tit_exp6 = sprintf ('');
tit_exp8 = sprintf ('');
tit_exp10 = sprintf ('');
tit_imp2 = sprintf ('');
tit_imp4 = sprintf ('');
tit_imp6 = sprintf ('');
tit_imp8 = sprintf ('');
tit_imp10 = sprintf ('');


%display explicit dispersion relations

subplot (3,2,1); surf (kdx, mdt, Vpexp); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_exp2);
xlabel ('k*dx (rad)'), ylabel('dt (ms)');

subplot (3,2,3); surf (kdx, mdt, Vpexp4); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_exp4);
xlabel ('k*dx (rad)'), ylabel('dt (ms)');

%subplot (3,1,3); surf (kdx, mdt, Vpexp6); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
%title (tit_exp6);
%xlabel ('k*dx (rad)'), ylabel('dt (ms)');

%subplot (3,1,3); surf (kdx, mdt, Vpexp8); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
%title (tit_exp8);
%xlabel ('k*dx (rad)'), ylabel('dt (ms)');

subplot (3,2,5); surf (kdx, mdt, Vpexp10); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_exp10);
xlabel ('k*dx (rad)'), ylabel('dt (ms)');

%display implicit dispersion relations
%figure
subplot (3,2,2); surf (kdx, mdt, Vp2); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_imp2);
xlabel ('k*dx (rad)'), ylabel('dt (ms)');

subplot (3,2,4); surf (kdx, mdt, Vp4); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_imp4);
xlabel ('k*dx (rad)'), ylabel('dt (ms)');

%subplot (3,1,3); surf (kdx, mdt, Vp6); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
%title (tit_imp6);
%xlabel ('k*dx (rad)'), ylabel('dt (ms)');

%subplot (3,1,3); surf (kdx, mdt, Vp8); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
%title (tit_imp8);
%xlabel ('k*dx (rad)'), ylabel('dt (ms)');

subplot (3,2,6); surf (kdx, mdt, Vp10); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
title (tit_imp10);
xlabel ('k*dx (rad)'), ylabel('dt (ms)');
