% display the pseudo-laplacian as a function of dt,v,dx,dz,kx,kz

echo off;
clear;
clc;

%load('blue_red_disp', 'blue_red_disp_map')
%set(0,'DefaultFigureColorMap',blue_red_disp_map);
set(0,'DefaultAxesFontSize',16);
set(0, 'DefaultFigurePosition', [100 100 1000 800]);
set(0, 'DefaultAxesYTickLabelMode', 'auto');
set(0, 'DefaultAxesXTickLabelMode', 'auto');
%set(0, 'DefaultAxesYTick', [0.785 1.57]);
%set(0, 'DefaultAxesYTickLabel', {'pi/4' ; 'pi/2'});
%set(0, 'DefaultAxesXTick', [0.785 1.57]);
%set(0, 'DefaultAxesXTickLabel', {'pi/4' ; 'pi/2'});


nx	= 30;
nz	= 30;
dx  = 10;
dz  = 10;
v   = 1500;
vb  = 2900;
vc	= 2200;
dt  = 0.001;

v2  = v^2;
vb2 = vb^2;
vc2 = vc^2;
dx2 = dx^2;
dz2 = dz^2;
dt2 = dt^2;

k		= pi/(nx*dx) : pi/(nx*dx) : pi/4;
%k		= pi/(nx*dx) : pi/(nx*dx) : pi;
kx		= k;
kz		= k;
kx2		= kx.^2;
kz2		= kz.^2;

k_first = k(1);
k_last  = k(size(k(:)));
k_last  = k_last(1);


for i = 1:size(kz2(:))
	for j = 1:size(kx2(:))
		kabs = sqrt (kx2(j) + kz2(i));
		pseudospec(i,j) = - kabs;
		pseudodiff (i,j)  = (2 * cos (v  * dt * kabs) - 2) / (v2  * dt2);
		pseudodiffb (i,j) = (2 * cos (vb * dt * kabs) - 2) / (vb2 * dt2);
		pseudodiffc (i,j) = (2 * cos (vc * dt * kabs) - 2) / (vc2 * dt2);
		interp_pseudodiff (i,j) = (pseudodiff (i,j) + pseudodiffb (i,j)) * 0.5;
	end
end

cax_min = min(min(pseudospec));
cax_max = max(max(pseudodiffb));

figure;
subplot (2,2,1);
surf (kx, kz, pseudodiff); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last k_first k_last]);  shading interp;
xlabel ('kx (rad)'), ylabel('kz (rad)'); title ('pseudo-differential v=1500');

subplot (2,2,2);
surf (kx, kz, pseudodiffc); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last k_first k_last]);  shading interp;
xlabel ('kx (rad)'), ylabel('kz (rad)'); title ('pseudo-differential v=2200');

subplot (2,2,3);
surf (kx, kz, pseudodiffb); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last k_first k_last]);  shading interp;
xlabel ('kx (rad)'), ylabel('kz (rad)'); title ('pseudo-differential v=2900');

subplot (2,2,4);
surf (kx, kz, interp_pseudodiff); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last k_first k_last]);  shading interp;
xlabel ('kx (rad)'), ylabel('kz (rad)'); title ('interpolated pseudo-differential v1=2900, v2=1500');

figure;
subplot (2,2,1);
surf (kx, kz, pseudospec); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last k_first k_last]);  shading interp;
xlabel ('kx (rad)'), ylabel('kz (rad)'); title ('pseudo spectral');

subplot (2,2,2);
surf (kx, kz, pseudodiff); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last k_first k_last]);  shading interp;
xlabel ('kx (rad)'), ylabel('kz (rad)'); title ('pseudo-differential v=1500');

subplot (2,2,3);
surf (kx, kz, pseudodiffc); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last k_first k_last]);  shading interp;
xlabel ('kx (rad)'), ylabel('kz (rad)'); title ('pseudo-differential v=2200');

subplot (2,2,4);
surf (kx, kz, pseudodiffb); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last k_first k_last]);  shading interp;
xlabel ('kx (rad)'), ylabel('kz (rad)'); title ('pseudo-differential v=2900');

%eps		= 0.1;

%for i = 1:size(dt(:))

	% Implicit schemes

	%imp_asin	= asin ( sqrt((4+eps)*alpha2(i) .* sin_k1 ./ (1 + 4 .*alpha2(i) .* sin_k1 )) );
%	imp_asin2	= asin ( sqrt(4*alpha2(i) .* sin_k1 ./ (1 + 4 .*alpha2(i) .* sin_k1 )) );
%	Vp2(i,:)	= 2 .* imp_asin2 ./ (dt(i) .* kabs .* v);

%end


%cax_min = disp_min;
%cax_max = disp_max;
%cax_min = min(min(Vp2));
%cax_max = max(max(Vpexp10));


%tit_exp2 = sprintf ('');


%display explicit dispersion relations

%subplot (3,2,1); surf (kx, mdt, Vpexp); view (0, 90); colorbar; caxis([cax_min cax_max]); axis([k_first k_last dt_first dt_last]);  shading interp;
%title (tit_exp2);
%xlabel ('k*dx (rad)'), ylabel('dt (ms)');

