program dprop2dhelimp4
	use sep
	use dpolydiv
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: tt, i, j, k, s, jjj
	real				:: dx, dz, ox, oz, dt, ot, eps_in, m_pi
	real(8)				:: eps, expnorm, impnorm
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, big_dt, cvel, cvel2, fmax
	integer				:: nx, nz, nt, srcx, srcz, nsnaps, pint, nx_norm
	integer				:: stat, ncoeffs
	type(dfilter)		:: expfilt, impfilt
	integer				:: c, cz_p1, cz_p2, cz_m1, cz_m2, cx_p1, cx_p2, cx_m1, cx_m2
	integer				:: max_exponent = 30
	logical				:: verb

	real, pointer, dimension(:) 		:: Pprint, coeffprint	! pressure field for printing
	real(8), pointer, dimension(:) 		:: P0, P			! pressure fields
	real(8), pointer, dimension(:) 		:: r				! temp for result side of implicit matrix equation
	real(8), pointer, dimension(:)		:: tmp1, tmp2, crc, fcoeff

	real(8), pointer, dimension(:)		:: a_t1, b_t1, c_t1
	real(8), pointer, dimension(:)		:: a_t2, b_t2, c_t2
	real(8)								:: a_t3, b_t3, c_t3

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()
	call sep_init()

	call from_param ("nx", nx)
	call from_param ("dx", dx)
	call from_param ("ox", ox)
	call from_param ("nz", nz)
	call from_param ("dz", dz)
	call from_param ("oz", oz)
	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("fmax", fmax)
	call from_param ("ncoeffs", ncoeffs)
	call from_param ("eps", eps_in)
	eps = eps_in
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)
	call from_param ("verb", verb)


	!########################
	! initializations
	!########################

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	dt2		= dt  * dt
	cvel2	= cvel * cvel

	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	! save modelling parameters
	call to_history ("n1", nz)
	call to_history ("d1", dz)
	call to_history ("o1", oz)
	call to_history ("label1", "z")
	call to_history ("n2", nx)
	call to_history ("d2", dx)
	call to_history ("o2", ox)
	call to_history ("label2", "x")
	call to_history ("n3", nsnaps)
	call to_history ("d3", printdt)
	call to_history ("o3", 0)
	call to_history ("label3", "t")
	
	!########################
	! allocate data arrays
	!########################

	allocate (Pprint	(nz*nx))
	allocate (P0		(nz*nx))
	allocate (P			(nz*nx))
	allocate (r			(nz*nx))
	allocate (a_t1		(nz*nx))
	allocate (b_t1		(nz*nx))
	allocate (c_t1		(nz*nx))
	allocate (a_t2		(nz*nx))
	allocate (b_t2		(nz*nx))
	allocate (c_t2		(nz*nx))

	!########################
	! read data
	!########################

	P0 = 0.0
	P  = 0.0

	!########################
	! propagation loop
	!########################

	a_t1	=  cvel2 * dt2 * idx2 / (4. * 12.)
	b_t1	=  16. * a_t1
	c_t1	=  -1. - 2. * 30. * a_t1
	a_t1	= -a_t1

	a_t2	=  2. * cvel2 * dt2 * idx2 / (4. * 12.)
	b_t2	=  16. * a_t2
	c_t2	=  2. - 2. * 30. * a_t2
	a_t2	= -a_t2

	a_t3	=  cvel2 * dt2 * idx2 / (4. * 12.)
	b_t3	=  -16. * a_t3
	c_t3	=  1. + 2. * 30. * a_t3


	! make 4th order filter
	impfilt = dimpfac4 (eps, c_t3, b_t3, b_t3, a_t3, a_t3, nz, ncoeffs)

	if (verb) then
		!===============================
		! test filter autocorrelation - see if we get the correct finite difference weights
		write (0,*), "c_t3 =", c_t3
		write (0,*), "b_t3 = ", b_t3
		write (0,*), "a_t3 = ", a_t3

		call to_history ("n1", size(impfilt%flt), "cfile")
		call to_history ("o1", 1, "cfile")
		call to_history ("d1", 1, "cfile")
	
		allocate (coeffprint (size(impfilt%flt)))
		coeffprint = impfilt%flt	
		if (srite ("cfile", coeffprint, size(coeffprint) * sizeof (coeffprint(1))) .eq. 0) call seperr ("error writing filter coefficients")
		deallocate (coeffprint)

		allocate (fcoeff (size(impfilt%flt) + 1))
		allocate (crc (size (fcoeff)))
	
		fcoeff (1) = 1.
		fcoeff (2 : size (impfilt%flt) + 1) = impfilt%flt
	
		call dcorrelate (fcoeff, fcoeff, crc, size(fcoeff))
		write (0,*), "correlation before norm = ", crc
		expnorm = (c_t3 + eps) / crc(1)
		crc = crc * expnorm
		write (0,*), "correlation after norm = ", crc
		write (0,*), "expnorm = ", expnorm
		deallocate (crc)
		deallocate (fcoeff)
		!===============================
	end if

	call dpolydiv_init (nz*nx, impfilt)

	!========================== get normalization factor for the filter created by dimpfac4
	allocate (tmp1 (nz*nx))
	allocate (tmp2 (nz*nx))
	tmp1 = .0
	tmp2 = .0

	tmp1 (nx/2 * nz + nz/2) = 1.0

	stat = dpolydiv_lop (.true., .false., tmp2, tmp1)
	stat = dpolydiv_lop (.false.,  .false., tmp2, tmp1)

	!impnorm = tmp1 (nx/2 * nz + nz/2) / c_t3
	impnorm = tmp1 (nx/2 * nz + nz/2) / (c_t3 + eps)

	if (verb) then
		write (0,*), "impulse before norm = ", tmp1 (nx/2 * nz + nz/2)
		tmp1 = tmp1 * impnorm
		write (0,*), "impulse after norm =", tmp1 (nx/2 * nz + nz/2)
		write (0,*), "impnorm = ", impnorm
	end if

	deallocate (tmp1)
	deallocate (tmp2)

	!==========================

	do tt = 1, nt

		if (tt * dt .lt. srcmaxt) then
			P (srcz + srcx*nz) = P (srcz + srcx*nz) + cvel2 * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
		end if

		! set up result array
		r = 0.

		do j = 3, nx-2
			do i = 3, nz-2
				c		= (j-1) * nz + i
				cx_p1	= (j  ) * nz + i
				cx_p2	= (j+1) * nz + i
				cx_m1	= (j-2) * nz + i
				cx_m2	= (j-3) * nz + i

				cz_p1	= (j-1) * nz + i + 1
				cz_p2	= (j-1) * nz + i + 2
				cz_m1	= (j-1) * nz + i - 1
				cz_m2	= (j-1) * nz + i - 2

				r ((j-1)*nz + i) = &
				    a_t2 (cz_m2) * P  (cz_m2) + b_t2 (cz_m1) * P  (cz_m1) + c_t2 (c) * P  (c) + b_t2 (cz_p1) * P  (cz_p1) + a_t2 (cz_p2) * P  (cz_p2) &	! tridiagonal  z
				  + a_t2 (cx_m2) * P  (cx_m2) + b_t2 (cx_m1) * P  (cx_m1)                     + b_t2 (cx_p1) * P  (cx_p1) + a_t2 (cx_p2) * P  (cx_p2) &	! off-diagonal x
				  + a_t1 (cz_m2) * P0 (cz_m2) + b_t1 (cz_m1) * P0 (cz_m1) + c_t1 (c) * P0 (c) + b_t1 (cz_p1) * P0 (cz_p1) + a_t1 (cz_p2) * P0 (cz_p2) &	! tridiagonal  z
				  + a_t1 (cx_m2) * P0 (cx_m2) + b_t1 (cx_m1) * P0 (cx_m1)                     + b_t1 (cx_p1) * P0 (cx_p1) + a_t1 (cx_p2) * P0 (cx_p2)	! off-diagonal x
			end do

		end do
		! apply polydiv to invert matrix
		P0 = 0. ! reuse P0
		stat = dpolydiv_lop (.true.,  .false., P0, r)
		stat = dpolydiv_lop (.false., .false., P0, r)
		r = r * impnorm

		P0 = P

		! move results into pressure field array. 
		! -----> Which boundaries here???????????????
		do j = 3, nx-2
			do i = 3, nz-2
				P ((j-1)*nz + i) = r ((j-1)*nz + i)
				if ( exponent (P((j-1)*nz + i)) > max_exponent ) then
					write (0,*), "divergence at time step ", tt, "value: ", P((j-1)*nz + i)
					goto 199
				end if
			end do
		end do

		! print at intervals
		if (mod (tt,pint) .eq. 0) then
			!write (0 ,10, ADVANCE="NO"), t*dt
			Pprint = P
			if (srite ("out", Pprint, nz * nx * sizeof (Pprint(1))) .eq. 0) call seperr ("error writing wave field")
		end if

	end do

199	continue

	deallocate (Pprint)
	deallocate (P0)
	deallocate (P)
	deallocate (r)
	deallocate (a_t1)
	deallocate (b_t1)
	deallocate (c_t1)
	deallocate (a_t2)
	deallocate (b_t2)
	deallocate (c_t2)

10	format (10(" ", f8.3))
20	format (10(" ", e12.3))

end program
