program make_coeffs_kol
	use sep
	use polydiv
	use prop2dutils
	use ccrosskolmog
	use chelix

	implicit none

	integer, external	:: sreed, srite
	integer				:: tt, i, j, k, s, jjj, ff
	real				:: dx, dz, ox, oz, dt, ot, eps, m_pi
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, big_dt, cvel, cvel2, impnorm, expnorm, fmax
	integer				:: nx, nz, nt, srcx, srcz, nsnaps, pint, nx_norm
	integer				:: stat, ncoeffs
	type(cfilter)		:: cimpfilt, fac1, fac2
	type(filter)		:: impfilt
	integer				:: max_exponent = 30
	logical				:: verb

	real, pointer, dimension(:)			:: tmp1, tmp2, tmp3
	real								:: a_t3, b_t3, c_t3
	complex, pointer, dimension(:)		:: ctmp

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()
	call sep_init()

	call from_param ("nx", nx, 1)
	call from_param ("dx", dx, 1.)
	call from_param ("ox", ox, 0.)
	call from_param ("nz", nz, 1)
	call from_param ("dz", dz, 1.)
	call from_param ("oz", oz, 0.)
	call from_param ("nt", nt, 1)
	call from_param ("dt", dt, 1.)
	call from_param ("ot", ot, 0.)
	call from_param ("fmax", fmax)
	call from_param ("ncoeffs", ncoeffs)
	call from_param ("eps", eps)
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)
	call from_param ("verb", verb)

	!########################
	! initializations
	!########################

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	dt2		= dt  * dt
	cvel2	= cvel * cvel

	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	a_t3 = cvel2 * dt2 * idx2 / 4.
	b_t3 =  1. + 2. * a_t3
	a_t3 = -a_t3

	if ( nx < ncoeffs) then
		write (0,*), "error: nx must be greater than number of filter coefficients"
		call exit()
	end if

	! make kolmogoroff filter 
	! fac1 and fac2 are the filter and it's adjoint
	call impfac1d_kol (b_t3, a_t3, nx, ncoeffs, fac1, fac2)
	impfilt = impfac1d (eps, b_t3, a_t3, nx, ncoeffs)


	!===============================
	! test filter autocorrelation - see if we get the correct finite difference weights
	write (0,*), ""
	write (0,*), "implicit FD coefficients:"
	write (0,*), "b_t3 = ", b_t3
	write (0,*), "a_t3 = ", a_t3

	allocate (ctmp (size(fac1%lag)))
	call ccorrelate (fac1%flt, fac2%flt, ctmp, size(ctmp))
	write (0,*), "correlation of real and complex of kolmogoroff ="
	write (0,*), ctmp

	allocate (tmp1 (size(ctmp)), tmp2 (size(ctmp)), tmp3 (size(ctmp)))
	tmp1 = fac1%flt
	tmp2 = fac2%flt
	call correlate (tmp1, tmp2, tmp3, size (tmp3))
	write (0,*), "correlation of real part only of kolmogoroff ="
	write (0,*), tmp3

	deallocate (tmp1, tmp2, tmp3, ctmp)

	allocate (tmp1 (size(impfilt%flt) + 1))
	allocate (tmp2 (size(tmp1)))

	tmp1 (1) = 1.
	tmp1 (2 : size (impfilt%flt) + 1) = impfilt%flt
	call correlate (tmp1, tmp1, tmp2, size(tmp1))
	expnorm = (b_t3 + eps) / tmp2(1)
	tmp2 = tmp2 * expnorm

	write (0,*), "correlation of wilson after normalization ="
	write (0,*), tmp2

	call to_history ("n1", 2 * size(fac1%flt), "cfile")
	call to_history ("o1", 1, "cfile")
	call to_history ("d1", 1, "cfile")
	call to_history ("esize", 8, "cfile")
	call to_history ("n1", size(fac1%lag), "lfile")
	call to_history ("o1", 1, "lfile")
	call to_history ("d1", 1, "lfile")
	call to_history ("n1", size(impfilt%flt), "cfile2")
	call to_history ("o1", 1, "cfile2")
	call to_history ("d1", 1, "cfile2")
	call to_history ("n1", size(impfilt%lag), "lfile2")
	call to_history ("o1", 1, "lfile2")
	call to_history ("d1", 1, "lfile2")

	if (srite ("cfile", fac1%flt, size(fac1%flt) * sizeof (fac1%flt(1))) .eq. 0) call seperr ("error writing filter coefficients")
	if (srite ("cfile", fac2%flt, size(fac2%flt) * sizeof (fac2%flt(1))) .eq. 0) call seperr ("error writing filter coefficients")
	if (srite ("lfile", fac1%lag, size(fac1%lag) * sizeof (fac1%lag(1))) .eq. 0) call seperr ("error writing filter lags")
	if (srite ("cfile2", impfilt%flt, size(impfilt%flt) * sizeof (impfilt%flt(1))) .eq. 0) call seperr ("error writing filter coefficients")
	if (srite ("lfile2", impfilt%lag, size(impfilt%lag) * sizeof (impfilt%lag(1))) .eq. 0) call seperr ("error writing lags coefficients")



!	if (verb) then
!		call polydiv_init (nz*nx, impfilt)
!	
!		allocate (tmp1 (nz*nx))
!		allocate (tmp2 (nz*nx))
!		tmp1 = .0
!		tmp2 = .0
!	
!		tmp1 (nx/2 * nz + nz/2) = 1.0
!	
!		stat = polydiv_lop (.true., .false., tmp2, tmp1)
!		stat = polydiv_lop (.false.,  .false., tmp2, tmp1)
!	
!		impnorm = tmp1 (nx/2 * nz + nz/2) / b_t3
!	
!		write (0,*), "impulse before norm = ", tmp1 (nx/2 * nz + nz/2)
!		tmp1 = tmp1 * impnorm
!		write (0,*), "impulse after norm =", tmp1 (nx/2 * nz + nz/2)
!		write (0,*), "impnorm = ", impnorm
!	
!		deallocate (tmp1)
!		deallocate (tmp2)
!	end if

end program
