program dprop2dhelimp4
	use sep
	use dpolydiv
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: tt, i, j, k, s, jjj
	real				:: dx, dz, ox, oz, dt, ot, eps_in, m_pi
	real(8)				:: eps, expnorm, impnorm
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, big_dt, cvel, cvel2, fmax
	integer				:: nx, nz, nt, srcx, srcz, nsnaps, pint, nx_norm
	integer				:: stat, ncoeffs
	type(dfilter)		:: expfilt, impfilt
	integer				:: c, cz_p1, cz_p2, cz_m1, cz_m2, cx_p1, cx_p2, cx_m1, cx_m2
	integer				:: max_exponent = 30
	logical				:: verb

	real, pointer, dimension(:) 		:: coeffprint	! pressure field for printing
	real(8), pointer, dimension(:)		:: tmp1, tmp2, crc, fcoeff

	real(8)								:: a_t3, b_t3, c_t3

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()
	call sep_init()

	call from_param ("nx", nx)
	call from_param ("dx", dx)
	call from_param ("ox", ox)
	call from_param ("nz", nz)
	call from_param ("dz", dz)
	call from_param ("oz", oz)
	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("fmax", fmax)
	call from_param ("ncoeffs", ncoeffs)
	call from_param ("eps", eps_in)
	eps = eps_in
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)
	call from_param ("verb", verb)


	!########################
	! initializations
	!########################

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	dt2		= dt  * dt
	cvel2	= cvel * cvel

	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	a_t3	=  cvel2 * dt2 * idx2 / (4. * 12.)
	b_t3	=  -16. * a_t3
	c_t3	=  1. + 2. * 30. * a_t3

	! make 4th order filter
	impfilt = dimpfac4 (eps, c_t3, b_t3, b_t3, a_t3, a_t3, nz, ncoeffs)

	!===============================
	! test filter autocorrelation - see if we get the correct finite difference weights
	write (0,*), ""
	write (0,*), "implicit FD coefficients:"
	write (0,*), "c_t3 =", c_t3
	write (0,*), "b_t3 = ", b_t3
	write (0,*), "a_t3 = ", a_t3

	call to_history ("n1", size(impfilt%flt), "cfile")
	call to_history ("o1", 1, "cfile")
	call to_history ("d1", 1, "cfile")
	call to_history ("n1", size(impfilt%lag), "lfile")
	call to_history ("o1", 1, "lfile")
	call to_history ("d1", 1, "lfile")

	allocate (coeffprint (size(impfilt%flt)))
	coeffprint = impfilt%flt	
	if (srite ("cfile", coeffprint, size(coeffprint) * sizeof (coeffprint(1))) .eq. 0) call seperr ("error writing filter coefficients")
	if (srite ("lfile", impfilt%lag, size(impfilt%lag) * sizeof (impfilt%lag(1))) .eq. 0) call seperr ("error writing filter lags")
	deallocate (coeffprint)

	if (verb) then
		allocate (fcoeff (size(impfilt%flt) + 1))
		allocate (crc (size (fcoeff)))
	
		fcoeff (1) = 1.
		fcoeff (2 : size (impfilt%flt) + 1) = impfilt%flt
	
		call dcorrelate (fcoeff, fcoeff, crc, size(fcoeff))
		write (0,*), "correlation before norm = ", crc
		expnorm = (c_t3 + eps) / crc(1)
		crc = crc * expnorm
		write (0,*), "correlation after norm = ", crc
		write (0,*), "expnorm = ", expnorm
		deallocate (crc)
		deallocate (fcoeff)
		!===============================
	
		call dpolydiv_init (nz*nx, impfilt)
	
		!========================== get normalization factor for the filter created by dimpfac4
		allocate (tmp1 (nz*nx))
		allocate (tmp2 (nz*nx))
		tmp1 = .0
		tmp2 = .0
	
		tmp1 (nx/2 * nz + nz/2) = 1.0
	
		stat = dpolydiv_lop (.true., .false., tmp2, tmp1)
		stat = dpolydiv_lop (.false.,  .false., tmp2, tmp1)
	
		!impnorm = tmp1 (nx/2 * nz + nz/2) / c_t3
		impnorm = tmp1 (nx/2 * nz + nz/2) / (c_t3 + eps)
	
		write (0,*), "impulse before norm = ", tmp1 (nx/2 * nz + nz/2)
		tmp1 = tmp1 * impnorm
		write (0,*), "impulse after norm =", tmp1 (nx/2 * nz + nz/2)
		write (0,*), "impnorm = ", impnorm
	
		deallocate (tmp1)
		deallocate (tmp2)
	end if

end program
