module opolydiv  
! Helix polynomial division with two filters made by kolmogoroff factorization
	use adj_mod
	use helix

	implicit none
	integer, private		:: nd 
	type (filter), private	:: aa, aa_inv 
	real, private, dimension (:), allocatable  :: tt 

contains

	subroutine opolydiv_init (nd_in, aa_in, aa_inv_in)
		integer			:: nd_in 
		type (filter)	:: aa_in, aa_inv_in 

		nd = nd_in 
		aa = aa_in 
		aa_inv = aa_inv_in 

		if (.not. allocated (tt)) then
			allocate (tt (nd)) 
		end if

	end subroutine

	function opolydiv_lop (adj, add, xx, yy) result(stat)
		integer            :: stat 
		logical,intent(in) :: adj, add 
		real,dimension(:)  :: xx, yy 

		call adjnull (adj, add, xx, yy)
		call opolydiv_lop2 (adj, add, xx, yy)

	    stat=0
	end function 

	subroutine opolydiv_lop2 (adj, add, xx, yy)
		logical, intent(in)	:: adj, add 
		real, dimension (:)	:: xx 
		real, dimension (:)	:: yy 
		integer				:: ia, ix, iy

		tt = 0.
		if (adj) then
			do ix = nd, 1, -1  
				tt (ix) = yy (ix)
				do ia = 2, size (aa_inv%lag)
					iy = ix + aa_inv%lag (ia)
					if (iy > nd) cycle
					tt (ix) = tt (ix) - aa_inv%flt (ia) * tt (iy)
				end do
			end do 
			xx = xx + tt
			!xx = xx + tt / aa_inv%flt(1)
		else
			do iy = 1, nd
				tt (iy) = xx (iy)
				do ia = 2, size (aa%lag)
					ix = iy - aa%lag (ia)
					if (ix < 1) cycle
					tt (iy) = tt (iy) - aa%flt (ia) * tt (ix)
				end do
			end do 
			yy = yy + tt
			!yy = yy + tt / aa%flt(1)
		end if
	end subroutine 

	subroutine opolydiv_close()
		deallocate (tt)
	end subroutine 

end module opolydiv
