program prop2dhel
	use sep
	use lapfac
	use helicon
	use polydiv

	implicit none

	integer, external	:: sreed, srite
	integer				:: t, i, j, k, s
	real				:: dx, dz, ox, oz, dt, ot, eps, m_pi
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, lapnorm, vmax, big_dt
	integer				:: nx, nz, nt, fmax, ncoeffs, srcx, srcz, nsnaps, pint
	integer				:: stat, mincols
	type(filter)		:: lapfilt

	real, pointer, dimension(:) 		:: Vpmodel	! velocity model
	real, pointer, dimension(:) 		:: P		! pressure field
	real, pointer, dimension(:) 		:: Pd		! pressure derivative field
	real, pointer, dimension(:) 		:: R1, R2	! spatial derivative fields

	!#######################
	! get input parameters
	!#######################
	call initpar()

	!call from_aux ('vmodel',  "n1", nz)
	nz = 20
	call from_aux ('vmodel',  "d1", dz)
	call from_aux ('vmodel',  "o1", oz)
	!call from_aux ('vmodel',  "n2", nx)
	nx = 20
	call from_aux ('vmodel',  "d2", dx)
	call from_aux ('vmodel',  "o2", ox)



	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("fmax", fmax)
	call from_param ("eps", eps)
	call from_param ("ncoeffs", ncoeffs)
	call from_param ("printdt", printdt)

	!########################
	! initializations
	!########################

	if ( nz .le. 2*ncoeffs ) then
		call seperr ("error: n1 must be greater than 2 * (number of filter coefficients)")
	end if

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	dt2		= dt  * dt
	!nsnaps	= nt * dt / printdt
	nsnaps	= 9
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax
	mincols	= 3

	lapfilt = lapfac2 (eps, nz, ncoeffs)
	call helicon_init (lapfilt)
	call polydiv_init (nx*nz, lapfilt) ! for implicit solution

!	write (0,*), "coeffs: ", lapfilt%flt
!	write (0,*), "lags: ", lapfilt%lag

	! save modelling parameters
	call to_history ("n1", nz)
	call to_history ("d1", dz)
	call to_history ("o1", oz)
	call to_history ("label1", "z")
	call to_history ("n2", nx)
	call to_history ("d2", dx)
	call to_history ("o2", ox)
	call to_history ("label2", "x")
	call to_history ("n3", nsnaps)
	call to_history ("d3", printdt)
	call to_history ("o3", 0)
	call to_history ("label3", "t")
	
	!########################
	! allocate data arrays
	!########################

	allocate (Vpmodel (nz*nx))
	allocate (P  (nz*nx))
	allocate (Pd  (nz*nx))
	allocate (R1  (nz*nx))
	allocate (R2  (nz*nx))

	!########################
	! read data
	!########################

	Vpmodel	= 0.0
	P		= 0.0
	Pd		= 0.0
	R1		= 0.0
	R2		= 0.0

	P (srcz + srcx*nz) = 1.0
	if (srite ("out", P, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")

	stat = polydiv_lop (.false., .false., P, R1)
	if (srite ("out", R1, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")

	stat = polydiv_lop (.true.,  .false., R2, R1)
	if (srite ("out", R2, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")

	stat = helicon_lop (.true., .false., R1, R2)
	if (srite ("out", R1, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")

	stat = helicon_lop (.false.,  .false., R1, P)
	if (srite ("out", P, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")

	stat = helicon_lop (.false.,  .false., P, R1)
	if (srite ("out", R1, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")

	stat = helicon_lop (.true., .false., R2, R1)
	if (srite ("out", R2, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")

	stat = polydiv_lop (.false., .false., R2, R1)
	if (srite ("out", R1, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")

	stat = polydiv_lop (.true.,  .false., P, R1)
	if (srite ("out", P, nz * nx * sizeof (P(0))) .eq. 0) call seperr ("error writing field")


	deallocate (Vpmodel)
	deallocate (P)
	deallocate (Pd)
	deallocate (R2)
	deallocate (R1)

end program
