program prop2dimp
	use sep
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: tt, i, j, k, s
	real				:: dx, dz, ox, oz, dt, ot, eps, m_pi
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, big_dt, cvel, cvel2, fmax
	integer				:: nx, nz, nt, srcx, srcz, nsnaps, pint
	integer				:: stat
	type(filter)		:: expfilt, impfilt

	real, pointer, dimension(:) 		:: P1, P0, P_1	! pressure field

	real			:: aa

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()

	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("nx", nx)
	call from_param ("dx", dx)
	call from_param ("ox", ox)
	call from_param ("fmax", fmax)
	call from_param ("eps", eps)
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)

	!########################
	! initializations
	!########################

	srcx	= 2
	idx		= 1. / dx
	idx2	= idx * idx
	dt2		= dt  * dt
	cvel2	= cvel * cvel
	aa		= cvel2 * dt2 * idx2


	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	! save modelling parameters
	call to_history ("n1", nx)
	call to_history ("d1", dx)
	call to_history ("o1", ox)
	call to_history ("label1", "x (m)")
	call to_history ("n2", nsnaps)
	call to_history ("d2", printdt)
	call to_history ("o2", 0)
	call to_history ("label2", "t (sec)")
	
	!########################
	! allocate data arrays
	!########################

	allocate (P1  (nx))
	allocate (P0 (nx))
	allocate (P_1  (nx))

	!########################
	! read data
	!########################

	P1  = 0.0
	P0  = 0.0
	P_1 = 0.0

	!########################
	! propagation loop
	!########################

	do tt = 1, nt

		if (tt * dt .lt. srcmaxt) then
			P0 (srcx) = P0 (srcx) + cvel2 * idx2 * dt2 * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			!P0 (srcx) = P0 (srcx) + cvel2 * idx2 * dt2 * srcfunc (fmax, tt*dt, srcmaxt * 0.1)
		end if

		do j = 2, nx-1
			P1 (j) = 2. * P0 (j) - P_1 (j) + aa * (P0 (j-1) - 2. * P0 (j) + P0 (j+1))
		end do

		do j = 2, nx-1
			P_1 (j) = P0 (j)
			P0  (j) = P1 (j)
		end do


		! print at intervals
		if (mod (tt,pint) .eq. 0) then
			!write (0,*), "snapshot at t= ", tt*dt
			if (srite ("out", P1, nx * sizeof (P1(1))) .eq. 0) call seperr ("error writing wave field")
		end if


	end do


	deallocate (P1)
	deallocate (P0)
	deallocate (P_1)

end program
