program prop1dhelimpkol
	use sep
	use polydiv
	use prop2dutils
	use ccrosskolmog
	use chelix
	use helix
	use opolydiv
	use ohelicon

	implicit none

	integer, external	:: sreed, srite
	integer				:: tt, i, j, k, s
	real				:: dx, dz, ox, oz, dt, ot, eps, m_pi, damp, impnorm, expnorm
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, big_dt, cvel, cvel2, fmax
	integer				:: nx, nz, nt, srcx, srcz, nsnaps, pint
	integer				:: stat, ncoeffs
	type(filter)		:: rfac1, rfac2
	type(cfilter)		:: fac1, fac2
	integer				:: max_exponent = 10
	logical				:: diverged = .false.

	real, pointer, dimension(:) 		:: P, P0	! pressure field
	real, pointer, dimension(:) 		:: r		! spatial derivative fields
	real, pointer, dimension(:)			:: tmp1, tmp2
	complex, pointer, dimension(:)		:: ctmp1

	real			:: aa
	real			:: ax_t1, bx_t1
	real			:: ax_t2, bx_t2
	real			:: ax_t3, bx_t3

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()

	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("nx", nx)
	call from_param ("dx", dx)
	call from_param ("ox", ox)
	call from_param ("fmax", fmax)
	call from_param ("eps", eps)
	call from_param ("damp", damp)
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)
	call from_param ("ncoeffs", ncoeffs)

	!########################
	! initializations
	!########################

	srcx	= 2
	idx		= 1. / dx
	idx2	= idx * idx
	dt2		= dt  * dt
	cvel2	= cvel * cvel


	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	! save modelling parameters
	call to_history ("n1", nx)
	call to_history ("d1", dx)
	call to_history ("o1", ox)
	call to_history ("label1", "x (m)")
	call to_history ("n2", nsnaps)
	call to_history ("d2", printdt)
	call to_history ("o2", 0)
	call to_history ("label2", "t (sec)")
	
	!########################
	! allocate data arrays
	!########################

	allocate (P  (nx))
	allocate (P0 (nx))
	allocate (r  (nx))

	!########################
	! read data
	!########################

	P = 0.0
	P0 = 0.0

	!########################
	! propagation loop
	!########################
	aa		= cvel2 * dt2 * idx2 / 4.

	ax_t1   =  aa
	bx_t1	=  -1. - 2. * aa

	ax_t2   =  2. * aa
	bx_t2	=  2. - 4. * aa

	ax_t3   =  - aa
	bx_t3	=  1. + 2. * aa

	write (0,*), "ax_t1 = ", ax_t1, "bx_t1 = ", bx_t1
	write (0,*), "ax_t2 = ", ax_t2, "bx_t2 = ", bx_t2
	write (0,*), "ax_t3 = ", ax_t3, "bx_t3 = ", bx_t3

	if ( nx .le. ncoeffs) then
		write (0,*), "error: nx must be greater than number of filter coefficients"
		call exit()
	end if


	! fac1 and fac2 are the filter and it's adjoint
	call impfac1d_kol (bx_t3, ax_t3, nx, ncoeffs, fac1, fac2)

	call allocatehelix (rfac1, ncoeffs)
	call allocatehelix (rfac2, ncoeffs)
	rfac1%flt = real (fac1%flt)
	rfac2%flt = real (fac2%flt)
	rfac1%lag = fac1%lag
	rfac2%lag = fac2%lag

	write (0,*), ""
	write (0,*), "filter1 coeffs: ", rfac1%flt
	write (0,*), ""
	write (0,*), "filter2 coeffs: ", rfac2%flt
	write (0,*), ""
	write (0,*), "filter1 lags: ",   rfac1%lag
	write (0,*), ""
	write (0,*), "filter2 lags: ",   rfac2%lag

	allocate (tmp1 (size(rfac1%flt)))
	call correlate (rfac1%flt, rfac2%flt, tmp1, size(tmp1))
	write (0,*), "real filter correlation ="
	write (0,*), tmp1
	deallocate (tmp1)

	!allocate (ctmp1 (size(fac1%flt)))
	!call ccorrelate (fac1%flt, fac2%flt, ctmp1, size(ctmp1))
	!write (0,*), "complex filter correlation ="
	!write (0,*), ctmp1
	!deallocate (ctmp1)

	call ohelicon_init (rfac1, rfac2)
	call opolydiv_init (nx, rfac1, rfac2)

	!==========================
	allocate (tmp1 (nx))
	allocate (tmp2 (nx))
	tmp1 = .0
	tmp2 = .0
	tmp1 (nx/2) = 1.0

	stat = opolydiv_lop (.true., .false., tmp2, tmp1)
	stat = opolydiv_lop (.false.,  .false., tmp2, tmp1)
	impnorm = tmp1 (nx/2) / (bx_t3 + eps)
	tmp1 = tmp1 * impnorm

	write (0,*), "spike after deconv:"
	write (0,*), tmp1

	tmp1 = .0
	tmp2 = .0
	tmp1 (nx/2) = 1.0

	stat = ohelicon_lop (.false., .false., tmp1, tmp2)
	stat = ohelicon_lop (.true., .false., tmp1, tmp2)
	expnorm = (bx_t3 + eps) / tmp1 (nx/2)
	tmp1 = tmp1 * expnorm

	write (0,*), "spike after conv:"
	write (0,*), tmp1

	deallocate (tmp1)
	deallocate (tmp2)

	!==========================

	do tt = 1, nt

		if (tt * dt .lt. srcmaxt) then
			P (srcx) = P (srcx) + cvel2 * dt2 * idx2 * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
		end if

		r  = 0.

		do j = 2, nx-1
			r (j) = ax_t2 * P (j+1) + bx_t2 * P (j) + ax_t2 * P (j-1) &
				   +ax_t1 * P0(j+1) + bx_t1 * P0(j) + ax_t1 * P0(j-1)
		end do

		! apply polydiv to invert matrix
		P0 = 0. ! reuse P0
		stat = opolydiv_lop (.true.,  .false., P0, r)
		stat = opolydiv_lop (.false., .false., P0, r)
		r = r * impnorm

		P0 = P
		P = r

		if (.not. diverged) then
			do j = 2, nx-1
				if (exponent(P (j)) > max_exponent .and. .not. diverged ) then
					write (0,*), "divergence at time ", tt*dt, "value: ", P(j)
					diverged = .true.
					!goto 199
				end if
			end do
		end if

		! print at intervals
		if (mod (tt, pint) .eq. 0) then
			!write (0,*), "snapshot at t= ", tt*dt
			if (srite ("out", P, nx * sizeof (P(1))) .eq. 0) call seperr ("error writing wave field")
		end if


	end do

199	continue

	deallocate (P)
	deallocate (P0)
	deallocate (r)

end program
