program prop2dimp
	use sep
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: tt, i, j, k, s
	real				:: dx, dz, ox, oz, dt, ot, eps, m_pi, damp
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, big_dt, cvel, cvel2, fmax, sideweight, centerweight
	integer				:: nx, nz, nt, srcx, srcz, nsnaps, pint
	integer				:: stat
	type(filter)		:: expfilt, impfilt

	real, pointer, dimension(:) 		:: P, P0	! pressure field
	real, pointer, dimension(:) 		:: r, res	! spatial derivative fields

	real			:: aa
	real			:: ax_t1, az_t1, bx_t1, bz_t1, cx_t1, cz_t1
	real			:: ax_t2, az_t2, bx_t2, bz_t2, cx_t2, cz_t2
	real			:: ax_t3, az_t3, bx_t3, bz_t3, cx_t3, cz_t3

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()

	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("nx", nx)
	call from_param ("dx", dx)
	call from_param ("ox", ox)
	call from_param ("fmax", fmax)
	call from_param ("eps", eps)
	call from_param ("damp", damp)
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)

	!########################
	! initializations
	!########################

	srcx	= 2
	idx		= 1. / dx
	idx2	= idx * idx
	dt2		= dt  * dt
	cvel2	= cvel * cvel


	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	! save modelling parameters
	call to_history ("n1", nx)
	call to_history ("d1", dx)
	call to_history ("o1", ox)
	call to_history ("label1", "x (m)")
	call to_history ("n2", nsnaps)
	call to_history ("d2", printdt)
	call to_history ("o2", 0)
	call to_history ("label2", "t (sec)")
	
	!########################
	! allocate data arrays
	!########################

	!allocate (Vpmodel (nz*nx))
	allocate (P  (nx))
	allocate (P0 (nx))
	allocate (r  (nx))
	allocate (res(nx))

	!########################
	! read data
	!########################

	P = 0.0
	P0 = 0.0

	!########################
	! propagation loop
	!########################

	!centerweight	= 2. + eps
	!sideweight		= 1.
	!aa		= cvel2 * dt2 * idx2 / (2 * sideweight + centerweight - eps)

	aa		= cvel2 * dt2 * idx2 / 4.

	ax_t1   =  aa
	bx_t1	=  -1. - 2. * aa
	cx_t1	=  aa

	ax_t2   =  2. * aa
	bx_t2	=  2. - 4. * aa 
	cx_t2	=  2. * aa

	ax_t3   =  - aa
	bx_t3	=  1. + 2. * aa
	cx_t3	=  - aa

	write (0,*), "ax_t1 = ", ax_t1, "bx_t1 = ", bx_t1
	write (0,*), "ax_t2 = ", ax_t2, "bx_t2 = ", bx_t2
	write (0,*), "ax_t3 = ", ax_t3, "bx_t3 = ", bx_t3

	do tt = 1, nt

		if (tt * dt .lt. srcmaxt) then
			P (srcx) = P (srcx) + cvel2 * dt2 * idx2 * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			!P (srcx) = P (srcx) + cvel2 * dt2 * idx2 * srcfunc (fmax, tt*dt, srcmaxt * 0.1)
		end if


		r(1)  = 0.
		r(nx) = 0.
		res   = 0.

		do j = 2, nx-1
			r (j) = ax_t2 * P (j+1) + bx_t2 * P (j) + cx_t2 * P (j-1) &
				   +ax_t1 * P0(j+1) + bx_t1 * P0(j) + cx_t1 * P0(j-1)
		end do

		call rtris_lab7 (nx, 1e+30, ax_t3, bx_t3, cx_t3, 1e+30, r, res) ! Zero value boundary = free boundary = no pressure = opposite phase
		!call rtris_lab7 (nx, -ax_t3, ax_t3, bx_t3, cx_t3, -cx_t3, r, res) ! Zero slope boundary = rigid boundary = no motion = same phase

		do j = 2, nx-1
			P0 (j) = P   (j)
			P  (j) = res (j)
		end do


		! print at intervals
		if (mod (tt,pint) .eq. 0) then
			!write (0,*), "snapshot at tt= ", tt*dt
			if (srite ("out", P, nx * sizeof (P(1))) .eq. 0) call seperr ("error writing wave field")
		end if


	end do


	deallocate (P)
	deallocate (P0)
	deallocate (r)
	deallocate (res)

end program
