program prop2dexp
	use sep
	use lapfac
	use helicon
	use createhelixmod
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: tt, i, j, k, s
	real				:: dx, dz, ox, oz, dt, ot, eps, m_pi
	real(4)				:: idx, idz, idx2, idz2, dt2, dt4
	real				:: printdt, srcmaxt, vmax, big_dt, cvel, cvel2, fmax
	integer				:: nx, nz, nt, ncoeffs, srcx, srcz, nsnaps, pint
	integer				:: stat, fdorder
	type(filter)		:: lapfilt
	integer				:: max_exponent = 30

	real(4), pointer, dimension(:) 		:: W1, W2, W3, W4, W5
	real(4), pointer, dimension(:) 		:: P_3, P_2, P_1, P0, P1, R, R2
	real(4), pointer, dimension(:) 		:: Vpmodel
	real(4), pointer, dimension(:)		:: tmpptr


	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()

	call from_aux ('vmodel',  "n1", nz)
	call from_aux ('vmodel',  "d1", dz)
	call from_aux ('vmodel',  "o1", oz)
	call from_aux ('vmodel',  "n2", nx)
	call from_aux ('vmodel',  "d2", dx)
	call from_aux ('vmodel',  "o2", ox)

	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("fmax", fmax)
	call from_param ("eps", eps)
	call from_param ("ncoeffs", ncoeffs)
	call from_param ("printdt", printdt)
	call from_param ("fdorder", fdorder)
	call from_param ("cvel", cvel)

	!########################
	! initializations
	!########################

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	dt2		= dt  * dt
	dt4		= dt2  * dt2 / 12.
	cvel2	= cvel * cvel

	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	call to_history ("n1", nz)
	call to_history ("d1", dz)
	call to_history ("o1", oz)
	call to_history ("label1", "z (m)")
	call to_history ("n2", nx)
	call to_history ("d2", dx)
	call to_history ("o2", ox)
	call to_history ("label2", "x (m)")
	call to_history ("n3", nsnaps)
	call to_history ("d3", printdt)
	call to_history ("o3", 0)
	call to_history ("label3", "t (sec)")
	
	!########################
	! allocate data arrays
	!########################

	allocate (Vpmodel	(nz*nx))
	allocate (W1 (nz*nx))
	allocate (W2 (nz*nx))
	allocate (W3 (nz*nx))
	allocate (R  (nz*nx))
	allocate (R2 (nz*nx))

	P_1 => W3
	P0  => W2
	P1	=> W1

	!########################
	! read data
	!########################

	P_1		= 0.0
	P0		= 0.0
	P1		= 0.0
	R		= 0.0
	R2		= 0.0

	if (sreed ("vmodel", Vpmodel, nz * nx * sizeof (Vpmodel(0)) ) .eq. 0) call seperr ("error reading velocity model")

	! numerical stability formula for 2D
	!vmax = GetVmax (Vpmodel, nz, nx)
	!big_dt = min (dx, dz) * sqrt (0.75 * (dx/dz)**2 / ((dx/dz)**2 + 1)) / vmax
	!big_dt = sqrt(0.5) * dx * vmax
	!write (0,*), "largest stable dt is ", big_dt

	Vpmodel  = Vpmodel * Vpmodel

	!########################
	! propagation loop
	!########################

	if (fdorder .eq. 410) then
		write (0,*), "spatial 10th order approximation and temporal 4th"

		allocate (W4 (nz*nx))
		allocate (W5 (nz*nx))
		P_3 => W5
		P_2 => W4
		P_3	= 0.0
		P_2	= 0.0

		do tt = 1, nt
			! apply source function
			if (tt * dt .lt. srcmaxt) then
				P_1 (srcz + srcx*nz) = P_1 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			end if

			do j = 5, nx-5
				do i = 6, nz-5

					P1 (i + j*nz) = -30. * P_1 (i + j*nz) + 16. * (P0 (i + j*nz) + P_2 (i + j*nz)) - P_3 (i + j*nz) - &
									12. * Vpmodel (i + j*nz) * dt2 * &
									(idz2 * (0.001587301 * (P_1 (i-5 + j*nz) + P_1 (i+5 + j*nz)) &
											-0.004960317 * (P_1 (i-4 + j*nz) + P_1 (i+4 + j*nz)) &
											+0.039682539 * (P_1 (i-3 + j*nz) + P_1 (i+3 + j*nz)) &
											-0.238095238 * (P_1 (i-2 + j*nz) + P_1 (i+2 + j*nz)) &
											+1.666666666 * (P_1 (i-1 + j*nz) + P_1 (i+1 + j*nz)) &
											-2.929761902 *  P_1 (i + j*nz)) + &
									 idx2 * (0.001587301 * (P_1 (i + (j-5)*nz) + P_1 (i + (j+5)*nz)) &
											-0.004960317 * (P_1 (i + (j-4)*nz) + P_1 (i + (j+4)*nz)) &
											+0.039682539 * (P_1 (i + (j-3)*nz) + P_1 (i + (j+3)*nz)) &
											-0.238095238 * (P_1 (i + (j-2)*nz) + P_1 (i + (j+2)*nz)) &
											+1.666666666 * (P_1 (i + (j-1)*nz) + P_1 (i + (j+1)*nz)) &
											-2.929761902 *  P_1 (i + j*nz)))

					if ( exponent (P1(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", t, "value: ", P1(i + j*nz)
						goto 199
					end if

				end do	
			end do	

			! cycle fields
			tmpptr	=> P_3
			P_3		=> P_2
			P_2		=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), t*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if
		end do

		deallocate (W4)
		deallocate (W5)


	else if (fdorder .eq. 44) then
		write (0,*), "spatial 4th order approximation and temporal 4th"

		allocate (W4 (nz*nx))
		allocate (W5 (nz*nx))
		P_3 => W5
		P_2 => W4
		P_3	= 0.0
		P_2	= 0.0

		do tt = 1, 5
			! apply source function
			if (tt * dt .lt. srcmaxt) then
				P0 (srcz + srcx*nz) = P0 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			end if

			! FREE surface condition (zero pressure)
			do j = 2, nx-2
				do i = 3, nz-2
					P1 (i + j*nz) = 2. * P0 (i + j*nz) - P_1 (i + j*nz) + &
									Vpmodel (i + j*nz) * dt2 * idz2 * &
									(-0.08333333 * (P0 (i-2 + j*nz) + P0 (i+2 + j*nz)) &
									 +1.33333333 * (P0 (i-1 + j*nz) + P0 (i+1 + j*nz)) &
									 -2.5        *  P0 (i + j*nz)) + &
									Vpmodel (i + j*nz) * dt2 * idx2 * &
									(-0.08333333 * (P0 (i + (j-2)*nz) + P0 (i + (j+2)*nz)) &
									 +1.33333333 * (P0 (i + (j-1)*nz) + P0 (i + (j+1)*nz)) &
									 -2.5        *  P0 (i + j*nz))

					if ( exponent (P_3(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", tt, "value: ", P_3(i + j*nz)
						goto 199
					end if

				end do
			end do

			! cycle fields
			tmpptr	=> P_3
			P_3		=> P_2
			P_2		=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), t*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if
		end do

		do tt = 5, nt
			! apply source function
			!if (tt * dt .lt. srcmaxt) then
			!	P_1 (srcz + srcx*nz) = P_1 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
				!P0 (srcz + srcx*nz) = P0 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			!end if

			do j = 2, nx-2
				do i = 3, nz-2

					P1 (i + j*nz) = -30. * P_1 (i + j*nz) + 16. * (P0 (i + j*nz) + P_2 (i + j*nz)) - P_3 (i + j*nz) - &
									Vpmodel (i + j*nz) * dt2 * & 
									(idz2 * (-1.  * (P_1 (i-2 + j*nz) + P_1 (i+2 + j*nz)) &
											 +16. * (P_1 (i-1 + j*nz) + P_1 (i+1 + j*nz)) &
											 -30. *  P_1 (i + j*nz)) + &
									 idx2 * (-1.  * (P_1 (i + (j-2)*nz) + P_1 (i + (j+2)*nz)) &
											 +16. * (P_1 (i + (j-1)*nz) + P_1 (i + (j+1)*nz)) &
											 -30. *  P_1 (i + j*nz)))

					if ( exponent (P_3(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", tt, "value: ", P_3(i + j*nz)
						goto 199
					end if

				end do	
			end do	

			! cycle fields
			tmpptr	=> P_3
			P_3		=> P_2
			P_2		=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), tt*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if

		end do

		deallocate (W4)
		deallocate (W5)

	else if (fdorder .eq. 310) then
		! diverges at dt=0.0093
		write (0,*), "spatial 10th order approximation and temporal 4th by Dablains 1/4th trick"
		do tt = 1, nt
			! apply source function
			if (tt * dt .lt. srcmaxt) then
				P0 (srcz + srcx*nz) = P0 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			end if

			do j = 5, nx-5
				do i = 6, nz-5
					R (i + j*nz) = Vpmodel (i + j*nz) * idz2 * &
									(  0.001587301 * (P0 (i-5 + j*nz) + P0 (i+5 + j*nz)) &
									  -0.004960317 * (P0 (i-4 + j*nz) + P0 (i+4 + j*nz)) &
									  +0.039682539 * (P0 (i-3 + j*nz) + P0 (i+3 + j*nz)) &
									  -0.238095238 * (P0 (i-2 + j*nz) + P0 (i+2 + j*nz)) &
									  +1.666666666 * (P0 (i-1 + j*nz) + P0 (i+1 + j*nz)) &
									  -2.929761902 *  P0 (i + j*nz)) + &
									Vpmodel (i + j*nz) * idx2 * &
									(  0.001587301 * (P0 (i + (j-5)*nz) + P0 (i + (j+5)*nz)) &
									  -0.004960317 * (P0 (i + (j-4)*nz) + P0 (i + (j+4)*nz)) &
									  +0.039682539 * (P0 (i + (j-3)*nz) + P0 (i + (j+3)*nz)) &
									  -0.238095238 * (P0 (i + (j-2)*nz) + P0 (i + (j+2)*nz)) &
									  +1.666666666 * (P0 (i + (j-1)*nz) + P0 (i + (j+1)*nz)) &
									  -2.929761902 *  P0 (i + j*nz))
				end do
			end do

			do j = 5, nx-5
				do i = 6, nz-5
					R2 (i + j*nz) = Vpmodel (i + j*nz) * idz2 * &
									(  0.001587301 * (R (i-5 + j*nz) + R (i+5 + j*nz)) &
									  -0.004960317 * (R (i-4 + j*nz) + R (i+4 + j*nz)) &
									  +0.039682539 * (R (i-3 + j*nz) + R (i+3 + j*nz)) &
									  -0.238095238 * (R (i-2 + j*nz) + R (i+2 + j*nz)) &
									  +1.666666666 * (R (i-1 + j*nz) + R (i+1 + j*nz)) &
									  -2.929761902 *  R (i + j*nz)) + &
									Vpmodel (i + j*nz) * idx2 * &
									(  0.001587301 * (R (i + (j-5)*nz) + R (i + (j+5)*nz)) &
									  -0.004960317 * (R (i + (j-4)*nz) + R (i + (j+4)*nz)) &
									  +0.039682539 * (R (i + (j-3)*nz) + R (i + (j+3)*nz)) &
									  -0.238095238 * (R (i + (j-2)*nz) + R (i + (j+2)*nz)) &
									  +1.666666666 * (R (i + (j-1)*nz) + R (i + (j+1)*nz)) &
									  -2.929761902 *  R (i + j*nz))

				end do
			end do

			do j = 2, nx-2
				do i = 3, nz-2
					P1 (i + j*nz) = 2 * P0 (i + j*nz) - P_1 (i + j*nz) + dt2 * R (i + j*nz) + dt4 * R2 (i + j*nz)

					if ( exponent (P1(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", tt, "value: ", P1(i + j*nz)
						goto 199
					end if
				end do
			end do

			! cycle fields
			tmpptr	=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), tt*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if
		end do

	else if (fdorder .eq. 34) then
		! diverges at dt=0.0106
		write (0,*), "spatial 4th order approximation and temporal 4th by Dablains 1/4th trick"
		do tt = 1, nt
			! apply source function
			if (tt * dt .lt. srcmaxt) then
				P0 (srcz + srcx*nz) = P0 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			end if

			do j = 2, nx-2
				do i = 3, nz-2
					R (i + j*nz) =  Vpmodel (i + j*nz) * idz2 * &
									(-0.08333333 * (P0 (i-2 + j*nz) + P0 (i+2 + j*nz)) &
									 +1.33333333 * (P0 (i-1 + j*nz) + P0 (i+1 + j*nz)) &
									 -2.5        *  P0 (i + j*nz)) + &
									Vpmodel (i + j*nz) * idx2 * &
									(-0.08333333 * (P0 (i + (j-2)*nz) + P0 (i + (j+2)*nz)) &
									 +1.33333333 * (P0 (i + (j-1)*nz) + P0 (i + (j+1)*nz)) &
									 -2.5        *  P0 (i + j*nz))
				end do
			end do

			do j = 2, nx-2
				do i = 3, nz-2
					!R2 (i + j*nz) = Vpmodel (i + j*nz) * idz2 * (R (i-1 + j*nz)   + R (i+1 + j*nz)   - 2.0 * R (i + j*nz)) + &
					!			     Vpmodel (i + j*nz) * idx2 * (R (i + (j-1)*nz) + R (i + (j+1)*nz) - 2.0 * R (i + j*nz))

					R2 (i + j*nz) = Vpmodel (i + j*nz) * idz2 * &
									(-0.08333333 * (R (i-2 + j*nz) + R (i+2 + j*nz)) &
									 +1.33333333 * (R (i-1 + j*nz) + R (i+1 + j*nz)) &
									 -2.5        *  R (i + j*nz)) + &
									Vpmodel (i + j*nz) * idx2 * &
									(-0.08333333 * (R (i + (j-2)*nz) + R (i + (j+2)*nz)) &
									 +1.33333333 * (R (i + (j-1)*nz) + R (i + (j+1)*nz)) &
									 -2.5        *  R (i + j*nz))
				end do
			end do

			do j = 2, nx-2
				do i = 3, nz-2
					P1 (i + j*nz) = 2 * P0 (i + j*nz) - P_1 (i + j*nz) + dt2 * R (i + j*nz) + dt4 * R2 (i + j*nz)

					if ( exponent (P1(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", tt, "value: ", P1(i + j*nz)
						goto 199
					end if
				end do
			end do

			! cycle fields
			tmpptr	=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), tt*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if
		end do

	else if (fdorder .eq. 32) then
		! diverges at dt=0.0122
		write (0,*), "spatial 2nd order approximation and temporal 4th by Dablains 1/4th trick"
		do tt = 1, nt
			! apply source function
			if (tt * dt .lt. srcmaxt) then
				P0 (srcz + srcx*nz) = P0 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			end if

			do j = 1, nx-1
				do i = 2, nz-1
					R (i + j*nz) = Vpmodel (i + j*nz) * (idz2 * (P0 (i-1 + j*nz)   + P0 (i+1 + j*nz)   - 2.0 * P0 (i + j*nz)) + &
								   						 idx2 * (P0 (i + (j-1)*nz) + P0 (i + (j+1)*nz) - 2.0 * P0 (i + j*nz)))
				end do
			end do

			do j = 1, nx-1
				do i = 2, nz-1
					R2 (i + j*nz) = Vpmodel (i + j*nz) * (idz2 * (R (i-1 + j*nz)   + R (i+1 + j*nz)   - 2.0 * R (i + j*nz)) + &
								    					  idx2 * (R (i + (j-1)*nz) + R (i + (j+1)*nz) - 2.0 * R (i + j*nz)))
				end do
			end do

			do j = 1, nx-1
				do i = 2, nz-1
					P1 (i + j*nz) = 2 * P0 (i + j*nz) - P_1 (i + j*nz) + dt2 * R (i + j*nz) + dt4 * R2 (i + j*nz)

					if ( exponent (P1(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", tt, "value: ", P1(i + j*nz)
						goto 199
					end if
				end do
			end do

			! cycle fields
			tmpptr	=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), tt*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if
		end do

	else if (fdorder .eq. 210) then
		! diverges at dt=0.0054
		write (0,*), "spatial 10th order approximation and temporal 2nd"
		do tt = 1, nt
			! apply source function
			if (tt * dt .lt. srcmaxt) then
				P0 (srcz + srcx*nz) = P0 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			end if

			! FREE surface condition (zero pressure)
			do j = 5, nx-5
				do i = 6, nz-5
					P1 (i + j*nz) = 2 * P0 (i + j*nz) - P_1 (i + j*nz) + &
									Vpmodel (i + j*nz) * dt2 * idz2 * &
									(  0.001587301 * (P0 (i-5 + j*nz) + P0 (i+5 + j*nz)) &
									  -0.004960317 * (P0 (i-4 + j*nz) + P0 (i+4 + j*nz)) &
									  +0.039682539 * (P0 (i-3 + j*nz) + P0 (i+3 + j*nz)) &
									  -0.238095238 * (P0 (i-2 + j*nz) + P0 (i+2 + j*nz)) &
									  +1.666666666 * (P0 (i-1 + j*nz) + P0 (i+1 + j*nz)) &
									  -2.929761902 *  P0 (i + j*nz)) + &
									Vpmodel (i + j*nz) * dt2 * idx2 * &
									(  0.001587301 * (P0 (i + (j-5)*nz) + P0 (i + (j+5)*nz)) &
									  -0.004960317 * (P0 (i + (j-4)*nz) + P0 (i + (j+4)*nz)) &
									  +0.039682539 * (P0 (i + (j-3)*nz) + P0 (i + (j+3)*nz)) &
									  -0.238095238 * (P0 (i + (j-2)*nz) + P0 (i + (j+2)*nz)) &
									  +1.666666666 * (P0 (i + (j-1)*nz) + P0 (i + (j+1)*nz)) &
									  -2.929761902 *  P0 (i + j*nz))

					if ( exponent (P1(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", tt, "value: ", P1(i + j*nz)
						goto 199
					end if

				end do
			end do

			! cycle fields
			tmpptr	=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), tt*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if
		end do

	else if (fdorder .eq. 24) then
		! diverges at dt=0.0061
		write (0,*), "spatial 4th order approximation and temporal 2nd"
		do tt = 1, nt
			! apply source function
			if (tt * dt .lt. srcmaxt) then
				P0 (srcz + srcx*nz) = P0 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			end if

			! FREE surface condition (zero pressure)
			do j = 2, nx-2
				do i = 3, nz-2
					P1 (i + j*nz) = 2 * P0 (i + j*nz) - P_1 (i + j*nz) + &
									Vpmodel (i + j*nz) * dt2 * idz2 * &
									(-0.08333333 * (P0 (i-2 + j*nz) + P0 (i+2 + j*nz)) &
									 +1.33333333 * (P0 (i-1 + j*nz) + P0 (i+1 + j*nz)) &
									 -2.5        *  P0 (i + j*nz)) + &
									Vpmodel (i + j*nz) * dt2 * idx2 * &
									(-0.08333333 * (P0 (i + (j-2)*nz) + P0 (i + (j+2)*nz)) &
									 +1.33333333 * (P0 (i + (j-1)*nz) + P0 (i + (j+1)*nz)) &
									 -2.5        *  P0 (i + j*nz))

					if ( exponent (P1(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", tt, "value: ", P1(i + j*nz)
						goto 199
					end if

				end do
			end do

			! cycle fields
			tmpptr	=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), tt*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if
		end do
	else
		! diverges at dt=0.007
		write (0,*), "spatial 2nd order approximation and temporal 2nd"
		do tt = 1, nt
			! apply source function
			if (tt * dt .lt. srcmaxt) then
				P0 (srcz + srcx*nz) = P0 (srcz + srcx*nz) + Vpmodel (srcz + srcx*nz) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
			end if

			! FREE surface condition (zero pressure)
			do j = 1, nx-1
				do i = 2, nz-1
					P1 (i + j*nz) = 2 * P0 (i + j*nz) - P_1 (i + j*nz) & 
									+ Vpmodel (i + j*nz) * dt2 * idz2 * (P0 (i-1 + j*nz) + P0 (i+1 + j*nz) - 2.0 * P0 (i + j*nz)) + &
									+ Vpmodel (i + j*nz) * dt2 * idx2 * (P0 (i + (j-1)*nz) + P0 (i + (j+1)*nz) - 2.0 * P0 (i + j*nz))

					if ( exponent (P1(i + j*nz)) > max_exponent ) then
						write (0,*), "divergence at time step ", tt, "value: ", P1(i + j*nz)
						goto 199
					end if

				end do
			end do

			! cycle fields
			tmpptr	=> P_1
			P_1		=> P0
			P0		=> P1
			P1		=> tmpptr

			! print at intervals
			if (mod (tt,pint) .eq. 0) then
				!write (0, 10, ADVANCE="NO"), tt*dt
				if (srite ("out", P0, nz * nx * sizeof (P0(1))) .eq. 0) call seperr ("error writing wave field")
			end if
		end do
	end if

199	continue

	deallocate (W1)
	deallocate (W2)
	deallocate (W3)
	deallocate (R)
	deallocate (R2)
	deallocate (Vpmodel)

10  format (10(" ", f8.3))


end program

!-----------------
!COMMON Example

!INTEGER :: count, array(1000)
!COMMON /awful / count, array

!Make a file (say awful.f90) containing:

!MODULE awful
!INTEGER :: count, array(1000)
!END MODULE awful
!USE a
