program prop2dimp
	use sep
	use lapfac
	use polydiv
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: tt, i, j, k, s, jjj
	real				:: dx, dz, ox, oz, dt, ot, eps, m_pi
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, vavg, big_dt, cvel, cvel2, impnorm, expnorm, fmax
	integer				:: nx, nz, nt, srcx, srcz, nsnaps, pint, nx_norm
	integer				:: stat, ncoeffs
	type(filter)		:: expfilt, impfilt
	real				:: alpha_in, beta_in
	integer				:: c, cz_p1, cz_m1, cx_p1, cx_m1
	integer				:: max_exponent = 40

	real, pointer, dimension(:) 		:: P0, P	! pressure fields
	real, pointer, dimension(:) 		:: r		! temp for result side of implicit matrix equation
	real, pointer, dimension(:)			:: tmp1, tmp2
	real, pointer, dimension(:) 		:: Vpmodel	! velocity model

	real, pointer, dimension(:)			:: aa, fcoeff, crc
	real, pointer, dimension(:)			:: a_t1, b_t1
	real, pointer, dimension(:)			:: a_t2, b_t2
	real								:: a_t3, b_t3

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()

	call from_aux ('vmodel',  "n1", nz)
	call from_aux ('vmodel',  "d1", dz)
	call from_aux ('vmodel',  "o1", oz)
	call from_aux ('vmodel',  "n2", nx)
	call from_aux ('vmodel',  "d2", dx)
	call from_aux ('vmodel',  "o2", ox)

	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("fmax", fmax)
	call from_param ("ncoeffs", ncoeffs)
	call from_param ("eps", eps)
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)

	!########################
	! initializations
	!########################

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	dt2		= dt  * dt
	cvel2	= cvel * cvel


	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	! save modelling parameters
	call to_history ("n1", nz)
	call to_history ("d1", dz)
	call to_history ("o1", oz)
	call to_history ("label1", "z(m)")
	call to_history ("n2", nx)
	call to_history ("d2", dx)
	call to_history ("o2", ox)
	call to_history ("label2", "x(m)")
	call to_history ("n3", nsnaps)
	call to_history ("d3", printdt)
	call to_history ("o3", 0)
	call to_history ("label3", "t(sec)")
	
	!########################
	! allocate data arrays
	!########################

	allocate (Vpmodel	(nz*nx))
	allocate (P0		(nz*nx))
	allocate (P			(nz*nx))
	allocate (r			(nz*nx))
	allocate (a_t1		(nz*nx))
	allocate (b_t1		(nz*nx))
	allocate (a_t2		(nz*nx))
	allocate (b_t2		(nz*nx))
	allocate (aa	(nz*nx))

	!########################
	! read data
	!########################

	P0 = 0.0
	P  = 0.0

	if (sreed ("vmodel", Vpmodel, nz * nx * sizeof (Vpmodel(0)) ) .eq. 0) call seperr ("error reading velocity model")

	! numerical stability formula for 2D
	vmax = GetVmax (Vpmodel, nz, nx)
	big_dt = 1. / (4*fmax)
	write (0,*), "largest stable dt is ", big_dt

	!########################
	! propagation loop
	!########################
	vavg	= GetVavg (Vpmodel, nz, nx)
	write (0,*), "average velocity is ", vavg
	vavg	= vavg * vavg
	Vpmodel = Vpmodel * Vpmodel

	aa = Vpmodel * dt2 * idx2 / 4.

	a_t1	= aa
	b_t1	= -1. - 4. * aa

	a_t2	=  2. * aa
	b_t2	=  2. - 8. * aa

	a_t3	= -1. * aa(1)
	b_t3	=  1. + 4. * aa(1)

	write (0,*), "b_t3 = ", b_t3
	write (0,*), "a_t3 = ", a_t3
	

	impfilt = impfac2 (eps, b_t3, a_t3, a_t3, nz, ncoeffs)

	do i = 1, size(impfilt%flt)
		write (0,*), "flt ", i, impfilt%flt(i), "at lag", impfilt%lag(i)
	end do

	!===============================
	! test filter autocorrelation - see if we get the correct finite difference weights
	allocate (fcoeff (size(impfilt%flt) + 1))
	allocate (crc (size (fcoeff)))

	fcoeff (1) = 1.
	fcoeff (2 : size (impfilt%flt) + 1) = impfilt%flt

	call correlate (fcoeff, fcoeff, crc, size(fcoeff))
	write (0,*), "correlation before norm = ", crc
	expnorm = (b_t3 + eps) / crc(1)
	crc = crc * expnorm
	write (0,*), "correlation after norm = ", crc
	write (0,*), "expnorm = ", expnorm
	deallocate (crc)
	deallocate (fcoeff)
	!===============================

	call polydiv_init (nz*nx, impfilt)

	!========================== get normalization factor for the filter created by impfac2
	allocate (tmp1 (nz*nx))
	allocate (tmp2 (nz*nx))
	tmp1 = .0
	tmp2 = .0

	tmp1 (nx/2 * nz + nz/2) = 1.0

	stat = polydiv_lop (.true., .false., tmp2, tmp1)
	stat = polydiv_lop (.false.,  .false., tmp2, tmp1)

	!impnorm = tmp1 (nx/2 * nz + nz/2) / b_t3
	impnorm = tmp1 (nx/2 * nz + nz/2) / (b_t3 + eps)
	write (0,*), "impulse before norm = ", tmp1 (nx/2 * nz + nz/2)
	tmp1 = tmp1 * impnorm
	write (0,*), "impulse after norm =", tmp1 (nx/2 * nz + nz/2)
	write (0,*), "impnorm = ", impnorm


	deallocate (tmp1)
	deallocate (tmp2)

	!==========================

	do tt = 1, nt


		if (tt * dt .lt. srcmaxt) then
			P (srcz + srcx*nz) = P (srcz + srcx*nz) + Vpmodel (srcz + srcx*nx) * dt2 * (idz2 + idx2) * srcfunc (fmax, tt*dt, srcmaxt * 0.5)
		end if

		! set up result array
		r = 0.

		do j = 2, nx-1
			do i = 2, nz-1
				c		= (j-1) * nz + i
				cx_p1	= (j  ) * nz + i
				cx_m1	= (j-2) * nz + i
				cz_p1	= (j-1) * nz + i + 1
				cz_m1	= (j-1) * nz + i - 1

				r ((j-1)*nz + i) = &
				    a_t2 (cz_m1) * P  (cz_m1) + b_t2 (c) * P  (c) + a_t2 (cz_p1) * P  (cz_p1) &	! tridiagonal  z
				  + a_t2 (cx_m1) * P  (cx_m1)                     + a_t2 (cx_p1) * P  (cx_p1) &	! off-diagonal x
				  + a_t1 (cz_m1) * P0 (cz_m1) + b_t1 (c) * P0 (c) + a_t1 (cz_p1) * P0 (cz_p1) &	! tridiagonal  z
				  + a_t1 (cx_m1) * P0 (cx_m1)                     + a_t1 (cx_p1) * P0 (cx_p1)		! off-diagonal x
			end do

		end do
		! apply polydiv to invert matrix
		P0 = 0. ! reuse P0
		stat = polydiv_lop (.true.,  .false., P0, r)
		stat = polydiv_lop (.false., .false., P0, r)
		r = r * impnorm


		P0 = P

		! move results into pressure field array. 
		! -----> Which boundaries here???????????????
		do j = 2, nx-1
			do i = 2, nz-1
				P ((j-1)*nz + i) = r ((j-1)*nz + i)
				if ( exponent (P((j-1)*nz + i)) > max_exponent ) then
					write (0,*), "divergence at time step ", tt, "value: ", P((j-1)*nz + i)
					goto 199
				end if
			end do
		end do

		! print at intervals
		if (mod (tt,pint) .eq. 0) then
			!write (0 ,10, ADVANCE="NO"), tt*dt
			if (srite ("out", P, nz * nx * sizeof (P(1))) .eq. 0) call seperr ("error writing wave field")
		end if


	end do

199	continue

	deallocate (Vpmodel)
	deallocate (P0)
	deallocate (P)
	deallocate (r)
	deallocate (a_t1)
	deallocate (b_t1)
	deallocate (a_t2)
	deallocate (b_t2)
	deallocate (aa)

10	format (10(" ", f8.3))
20	format (10(" ", e12.3))

end program
