program prop2dimp
	use sep
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: t, i, j, k, s
	real				:: dx, dz, ox, oz, dt, ot, eps, m_pi
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, big_dt, cvel, cvel2, fmax
	integer				:: N, nx, nz, nt, srcx, srcz, nsnaps, pint
	integer				:: stat
	type(filter)		:: expfilt, impfilt

	real, pointer, dimension(:) 		:: P, P0	! pressure field
	real, pointer, dimension(:) 		:: r	! spatial derivative fields
	real, pointer, dimension(:) 		:: PP	! implicit FD matrix

	real			:: aax, aaz
	real			:: ax_t1, az_t1, b_t1
	real			:: ax_t2, az_t2, b_t2
	real			:: ax_t3, az_t3, b_t3

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()

	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("nx", nx)
	call from_param ("dx", dx)
	call from_param ("ox", ox)
	call from_param ("nz", nz)
	call from_param ("dz", dz)
	call from_param ("oz", oz)
	call from_param ("fmax", fmax)
	call from_param ("eps", eps)
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)

	!########################
	! initializations
	!########################

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	dt2		= dt  * dt
	cvel2	= cvel * cvel


	nsnaps	= nt * dt / printdt
	pint	= (printdt + 1e-4) / dt
	srcmaxt = 2 * m_pi / fmax

	! save modelling parameters
	call to_history ("n1", nz)
	call to_history ("d1", dz)
	call to_history ("o1", oz)
	call to_history ("label1", "z (m)")
	call to_history ("n2", nx)
	call to_history ("d2", dx)
	call to_history ("o2", ox)
	call to_history ("label2", "x (m)")
	call to_history ("n3", nsnaps)
	call to_history ("d3", printdt)
	call to_history ("o3", 0)
	call to_history ("label3", "t (sec)")
	
	!########################
	! allocate data arrays
	!########################

	N = nz * nx

	allocate (P  (N))
	allocate (P0 (N))
	allocate (r  (N))
	allocate (PP (N * N))

	!########################
	! read data
	!########################

	P  = 0.0
	P0 = 0.0

	!########################
	! propagation loop
	!########################
	aax		= cvel2 * dt2 * idx2 / 3
	aaz		= cvel2 * dt2 * idz2 / 3

	ax_t1   =  aax
	az_t1   =  aaz
	b_t1	=  -1. - 2. * aaz - 2. * aax

	ax_t2   =  aax
	az_t2   =  aaz
	b_t2	=  2. - 2. * aaz - 2. * aax

	ax_t3   =  - aax
	az_t3   =  - aaz
	b_t3	=  1. + 2. * aaz + 2. * aax

	write (0,*), "az_t1 = ", az_t1, "b_t1 = ", b_t1, "ax_t1 = ", ax_t1
	write (0,*), "az_t2 = ", az_t2, "b_t2 = ", b_t2, "ax_t2 = ", ax_t2
	write (0,*), "az_t3 = ", az_t3, "b_t3 = ", b_t3, "ax_t3 = ", ax_t3

	do t = 1, nt

		if (t * dt .lt. srcmaxt) then
			P (srcz + srcx*nz) = P (srcz + srcx*nz) + cvel2 * dt2 * (idz2 + idx2) * srcfunc (fmax, t*dt, srcmaxt * 0.5)
		end if

		! create soluton vector for implicit solver (Ax=r)
		r = 0.

		do j = 2, nx-1
			do i = 2, nz-1
				r ((j-1)*nz + i) = &
					+ az_t2 * P  ((j-1)*nz + i - 1) + b_t2 * P  ((j-1)*nz + i) + az_t2 * P  ((j-1)*nz + i + 1) &
					+ ax_t2 * P  ((j-2)*nz + i)								   + ax_t2 * P  ((j)*nz + i) &
					+ az_t1 * P0 ((j-1)*nz + i - 1) + b_t1 * P0 ((j-1)*nz + i) + az_t1 * P0 ((j-1)*nz + i + 1) &
					+ ax_t1 * P0 ((j-2)*nz + i)								   + ax_t1 * P0 ((j)*nz + i)
			end do
		end do

		! create coefficient matrix of pressure field for implicit solver
		! NOTE - we are using wrap around over the z axis = helical boundary condition.
		! The off-diagonal elements always have a value
		! for nz=4 nx=2:
		! ---------------------------------------
		! |	b	az	0	0	|	ax	0	0	0	|
		! |	az	b	az	0	|	0	ax	0	0	|
		! |	0	az	b	az	|	0	0	ax	0	|
		! |	0	0	az	b	|	az	0	0	ax	|
		! | -  -  -  -  -  -  -  -  -  -  -  -  |
		! |	ax	0	0	az	|	b	az	0	0	|
		! |	0	ax	0	0	|	az	b	az	0	|
		! |	0	0	ax	0	|	0	az	b	az	|
		! |	0	0	0	ax	|	0	0	az	b	|
		! ---------------------------------------
		PP = 0.
	
		do k = 1, N - nz
			PP ((k-1) * N + k + nz)  = ax_t3
		end do
		do k = 1, N-1
			PP ((k-1) * N + k + 1 )  = az_t3
		end do
		do k = 1, N
			PP ((k-1) * N + k     )  = b_t3
		end do
		do k = 1 + 1, N
			PP ((k-1) * N + k - 1 )  = az_t3
		end do
		do k = 1 + nz, N
			PP ((k-1) * N + k - nz)  = ax_t3
		end do

!		if ( t .eq. 40 )	then
!			write (0,*), "--- BEFORE ---"
!			write (0,20), PP
!			write (0,*), " "
!			write (0,30), r
!		end if

		call gauss_elim_square_1d (N, PP, r)

!		if ( t .eq. 40 )	then
!			write (0,*), "--- AFTER ---"
!			write (0,20), PP
!			write (0,*), " "
!			write (0,30), r
!		end if

		P0 = P

		! move results into pressure field array. 
		! -----> Which boundaries here???????????????
		do j = 2, nx-1
			do i = 2, nz-1
				P ((j-1)*nz + i) = r ((j-1)*nz + i)
			end do
		end do



		! print at intervals
		if (mod (t,pint) .eq. 0) then
			write (0,10), t*dt
			if (srite ("out", P, N * sizeof (P(1))) .eq. 0) call seperr ("error writing wave field")
		end if


	end do

	deallocate (P)
	deallocate (P0)
	deallocate (r)
	deallocate (PP)

10	format (f5.3)
20	format (25(" ", f6.3))
30	format (25(" ", e12.3))

end program
