program prop2dimp
	use sep
	use lapfac
	use polydiv
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: i, j, k, s, jjj
	real				:: t, dx, dz, ox, oz, dt, ot, eps, oeps, deps, m_pi
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, vavg, big_dt, cvel, cvel2, impnorm, expnorm, fmax
	integer				:: nx, nz, nt, neps, srcx, srcz, nsnaps, pint, nx_norm
	integer				:: stat, ncoeffs
	type(filter)		:: expfilt, impfilt, impfilt_prev

	real, pointer, dimension(:)			:: tmpfilt

	real, pointer, dimension(:)			:: crc, fcoeff
	real								:: a_t3, b_t3, c_t3

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()

	call from_param ("nz", nz)
	call from_param ("dz", dz)
	call from_param ("oz", oz)
	call from_param ("nx", nx)
	call from_param ("dx", dx)
	call from_param ("ox", ox)

	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("ncoeffs", ncoeffs)
	call from_param ("oeps", oeps)
	call from_param ("neps", neps)
	call from_param ("deps", deps)
	call from_param ("cvel", cvel)

	!########################
	! initializations
	!########################

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	cvel2	= cvel * cvel

	! save modelling parameters
	call to_history ("n1", ncoeffs*4)
	call to_history ("d1", 1)
	call to_history ("o1", 1)
	call to_history ("label1", "coeffs")
	call to_history ("n2", neps)
	call to_history ("d2", deps)
	call to_history ("o2", oeps)
	call to_history ("label2", "eps")
	call to_history ("n3", nt)
	call to_history ("d3", dt)
	call to_history ("o3", ot)
	call to_history ("label3", "dt")
	
	! 4th spatial order FD weights

	allocate (tmpfilt (ncoeffs*4))

	do t = ot, ot + dt * nt, dt
		dt2 = t * t

		! 4th spatial order FD weights
		a_t3	=  cvel2 * dt2 * idx2 / (4. * 12.)
		b_t3	=  -16. * a_t3
		c_t3	=  1. + 2. * 30. * a_t3


		impfilt_prev = impfac4 (oeps, c_t3, b_t3, b_t3, a_t3, a_t3, nz, ncoeffs)

		do eps = oeps + deps, oeps + neps * deps, deps

			impfilt = impfac4 (eps, c_t3, b_t3, b_t3, a_t3, a_t3, nz, ncoeffs)
			tmpfilt = impfilt%flt / impfilt_prev%flt
			!tmpfilt = impfilt%flt
			impfilt_prev = impfilt

			if (srite ("out", tmpfilt, size(tmpfilt) * sizeof (tmpfilt(1))) .eq. 0) call seperr ("error writing filter coefficients")

		end do

	end do

end program
