program prop2dimp
	use sep
	use lapfac
	use polydiv
	use prop2dutils

	implicit none

	integer, external	:: sreed, srite
	integer				:: t, i, j, k, s, ie
	real				:: dx, dz, ox, oz, dt, ot, eps, deps, c_eps, m_pi, c_t
	real				:: idx, idz, idx2, idz2, dt2
	real				:: printdt, srcmaxt, vmax, vavg, big_dt, cvel, cvel2, impnorm, expnorm, fmax
	integer				:: nx, nz, nt, srcx, srcz, nsnaps, pint, nx_norm, neps
	integer				:: stat, ncoeffs
	type(filter)		:: expfilt, impfilt

	real, pointer, dimension(:)			:: tmp1, tmp2

	real								:: aa
	real								:: a_t3, b_t3

	m_pi = 2 * acos(0.)

	!#######################
	! get input parameters
	!#######################
	call initpar()

	call from_param ("nz", nz)
	call from_param ("dz", dz)
	call from_param ("oz", oz)
	call from_param ("nx", nx)
	call from_param ("dx", dx)
	call from_param ("ox", ox)

	call from_param ("nt", nt)
	call from_param ("dt", dt)
	call from_param ("ot", ot)
	call from_param ("fmax", fmax)
	call from_param ("ncoeffs", ncoeffs)
	call from_param ("eps", eps)
	call from_param ("neps", neps)
	call from_param ("deps", deps)
	call from_param ("printdt", printdt)
	call from_param ("cvel", cvel)
	

	!########################
	! initializations
	!########################

	srcx	= nx / 2
	srcz	= nz / 2
	idx		= 1. / dx
	idz		= 1. / dz
	idx2	= idx * idx
	idz2	= idz * idz
	cvel2	= cvel * cvel

	! save modelling parameters
	call to_history ("n1", nz*nx)
	call to_history ("d1", dz)
	call to_history ("o1", oz)
	call to_history ("label1", "x,z")
	call to_history ("n2", neps)
	call to_history ("d2", deps)
	call to_history ("o2", eps)
	call to_history ("label2", "eps")
	
	!########################
	! allocate data arrays
	!########################
	allocate (tmp1 (nz*nx))
	allocate (tmp2 (nz*nx))


	write (0,*), "b_t3 = ", b_t3, "a_t3 = ", a_t3

	do t = 0, nt-1
		c_t		= ot + t * dt
		dt2		= c_t  * c_t
		aa		= cvel2 * dt2 * idx2 / 4.
		a_t3	= -1. * aa
		b_t3	=  1. + 4. * aa
	
		impfilt = impfac2 (eps, b_t3, a_t3, a_t3, nz, ncoeffs)
	
		!write (0,*), ""
		!write (0,*), "impfilt coeffs: ", impfilt%flt
		!write (0,*), ""
		!write (0,*), "impfilt lags: ",   impfilt%lag
	
		write (0,*), "dt = ", c_t, "impfilt coeff lag 1: ", impfilt%flt(1)

		call helicon_init (impfilt)
		call polydiv_init (nz*nx, impfilt)
	
		!========================== get normalization factor for the filter created by impfac2
		tmp1 = .0
		tmp2 = .0
	
		tmp1 (nx/2 * nz + nz/2) = 1.0
	
		!======
		stat = helicon_lop (.false., .false., tmp1, tmp2)
		stat = helicon_lop (.true.,  .false., tmp1, tmp2)
		expnorm = (b_t3 + eps) / tmp1 (nx/2 * nz + nz/2)
		tmp1 = tmp1 * expnorm

		if (srite ("out", tmp1, nz * nx * sizeof (tmp1(1))) .eq. 0) call seperr ("error writing wave field")

		!write (0,20, ADVANCE="NO"), tmp1 (nx/2 * nz + nz/2)
		!write (0,*), "expnorm = ", expnorm
		!======
	
		!======
		!stat = polydiv_lop (.true., .false., tmp2, tmp1)
		!stat = polydiv_lop (.false.,  .false., tmp2, tmp1)
		!impnorm = tmp1 (nx/2 * nz + nz/2) / b_t3
		!impnorm = tmp1 (nx/2 * nz + nz/2) / (b_t3 + eps)
		!tmp1 = tmp1 * impnorm
	
		!if (srite ("out", tmp1, nz * nx * sizeof (tmp1(1))) .eq. 0) call seperr ("error writing wave field")
	
		!write (0,20, ADVANCE="NO"), tmp1 (nx/2 * nz + nz/2)
		!write (0,*), "impnorm = ", impnorm

		call polydiv_close()	
	
		!==========================
	end do

	deallocate (tmp1)
	deallocate (tmp2)

10	format (10(" ", f8.3))
20	format (10(" ", e12.3))

end program
